#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# coding: utf-8                                                                      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#

# -----------------------------------------------------------------------------------
# If you want to stress-test your NAS, open a terminal session for each CPU-Thread and enter:
# -->> cat /dev/zero > /dev/null
# This creates a 100% load for each CPU thread.
# -----------------------------------------------------------------------------------

import time
nTime = time.time()

import sys
import os
import threading
import signal

sys.path.append( os.path.dirname(os.path.abspath(sys.argv[0])) + "/gui")
sys.path.append( os.path.dirname(os.path.abspath(sys.argv[0])) + "/lib")

import asset
import basics   as io
import lib

import Init
import Oid
import Memory
import Sensors
import Sys
import Status
import Csv
import Flow

# -----------------------------------------------------------------------------------
# --- Redirect stderr to "ERROR" ...
if not sys.stdin.isatty() :
   sys.stderr = open( asset.ERROR, 'a' )

# -----------------------------------------------------------------------------------
AvrDebug  = False
RunOne    = False
DSM7      = False
AvrSmart  = False

# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------

# -----------------------------------------------------------------------------------
# --- Check if an argument has been passed
for iArg in sys.argv[1:] :
   while lib.switch( iArg ) :
      if lib.case( 'debug' ) :
         AvrDebug  = True
         sys.argv.remove( iArg )
         break
      if lib.case( 'init'  ) :
         AvrDebug  = True
         RunOne    = True
         sys.argv.remove( iArg )
         break
      if lib.case( 'dsm7'  ) :
         DSM7      = True
         sys.argv.remove( iArg )
         break
      if lib.case( 'smart'  ) :
         AvrSmart  = True
         sys.argv.remove( iArg )
         break
      # -----------------------------------------------------------------------------
      lib.Error( 'Unknown ARG "{0}" has passed, AvrLogger was canceled ...'.format( iArg ) )
      sys.exit()

# -----------------------------------------------------------------------------------
# --- Init Language for AvrLogger ...
asset.msg = lib.InitLanguage( 'admin', asset.avrLangFile )
io.ioWrite( str( int( time.time() ) ), asset.avrUpTimeFile )

# -----------------------------------------------------------------------------------
# --- Prevent the Deamon from being executed more than once ... 
# -----------------------------------------------------------------------------------
if not AvrDebug :
   # --- Discover all parent instances of "AvrLogger.py" ...
   # bash --> ps ax | grep -v grep | grep AvrLogger.py
   if len( lib.GetPPidOf( asset.SpkName + '.py' ) ) > 1 :
      # -----------------------------------------------------------------------------
      # --- If more than one instance was found ...
      lib.Error( asset.msg['errDouble'] )
      sys.exit()
   else :
      # -----------------------------------------------------------------------------
      io.ioWrite( str( os.getpid() ), asset.avrPidFile, opt=False )

# -----------------------------------------------------------------------------------
# --- Used signals between Daemon and GUI : Catch some Signals ...
signal.signal( 10, Init.ConfigUpdate )    # Signal from GUI (CGI) - Config changed
signal.signal( 16, Init.SvgUpdate    )    # Signal from cronjob   - SvgTick has expired, update plot
signal.signal( 30, Init.SvgArchive   )    # Signal from cronjob   - Archive timer has expired, archive plot

# -----------------------------------------------------------------------------------

# -----------------------------------------------------------------------------------
# --- Init locking Files (Flags) ....
# -----------------------------------------------------------------------------------
io.ioRm( asset.f_CrnFlag    )
io.ioRm( asset.f_AvrLock    )
io.ioRm( asset.f_SenLock    )
io.ioRm( asset.f_SvgLock    )
io.ioRm( asset.f_Heartbeat  )

io.ioRm( asset.f_OidLock    )
io.ioRm( asset.f_OidFlag    )

io.ioRm( asset.f_UpgradePKG )
io.ioRm( asset.f_avrStatus  )
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
Init.init( AvrDebug=AvrDebug )

# -----------------------------------------------------------------------------------
if RunOne :

   """
   # -------------------------------------------------------------------------------
   start = time.time()
   # -------------------------------------------------------------------------------
   print( 'Time : %.6f' % ( time.time() - start ) )

   # -------------------------------------------------------------------------------
   print ( f'USV IP   : {asset.AvrConf[ "UsvSlaveServer" ]}' )
   print ( f'USV Port : {asset.nutDefaultHostPort}' )
   sRet = Sensors.Get_UsvSock( asset.AvrConf[ 'UsvSlaveServer' ], asset.nutDefaultHostPort )

   for sKey in sRet :
      print ( f'{sKey} = {sRet[sKey]}' )
   # eSuffix  = [ 'Pri1LabelSuffix', 'Sec1LabelSuffix', 'Pri2LabelSuffix', 'Sec2LabelSuffix' ]

   iVoltage = sRet.get( 'input.voltage', 0 )
   print ( f'Voltage : {iVoltage}' )

   iID = 1
   Status.AlertTemp[iID] = 30.5
   Status.AlertMax[iID]  = 31.5
   Status.ThresholdPTCnotifyHigh( iID, '°C' )
   """

#  print ( f'AlertList = {Status.AlertList}' )
#
#  for j in Status.AlertList :
#
#     Suffix = asset.AvrConf[ eSuffix[ asset.BaseNum[ j ] ] ]
#     print ( f'Suffix ({Suffix}) for ID {j}' )

   # -------------------------------------------------------------------------------
   print ( asset.msg['dbgRunOnce'] + ' Time spend : %.4f' % ( time.time() - nTime) )
   sys.exit()

# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
if AvrDebug : Sys.Out_InitStat()
# -----------------------------------------------------------------------------------
# --- Main[endless]loop
# -----------------------------------------------------------------------------------
while True:

   if AvrDebug and asset.oid5 and asset.sys_DivCount != 0 : Sys.Out_TimeStat( asset.sys_DivCount )

   # --------------------------------------------------------------------------------
   # --- Wait until next Tick ...
   while time.time() <= asset.LoopTime : time.sleep( asset.LoopTime - time.time() ) 

   asset.OutStart = time.time()
   asset.LoopTime = time.time() + asset.sys_Tick
   # --- END Wait -------------------------------------------------------------------

   # --------------------------------------------------------------------------------
   # --- Call / Run asynchronous "OID"-Task ...
   if not asset.OIDpid.is_alive():
      asset.OIDpid = threading.Thread( target=Oid.Pick, args=( 0, asset.dOID, ) )
      asset.OIDpid.start()

   # --------------------------------------------------------------------------------

   # --------------------------------------------------------------------------------
   asset.LastSMA = ( asset.LastSMA + 1 ) % asset.MaxSMA
   # --------------------------------------------------------------------------------

   # --------------------------------------------------------------------------------
   # --- Request / collect all values ...
   Sensors.Get_CPU ()      # --- CPU-Temperature
   Sensors.Get_Load()      # --- CPU-Load
   Sensors.Get_Fan ()      # --- FAN-Values (PWM-Values) in percent (%)
   Sensors.Get_i2c ()      # --- i2c Form and average the average temperature
   Sensors.Get_Disk()      # --- Disk I/O statistics
   Sensors.Get_Eth ()      # --- Network I/O statistics
   Sensors.Get_Oid ()      # --- OID-Data
   Sensors.Get_Nut ()      # --- UPS Values ( Load / Discharge )
   Memory.Get_RAM  ()      # --- RAM usage
   # --------------------------------------------------------------------------------

   # --- Summe der Sensoren-Werte bilden ...
   for j in asset.BaseListSMA :
      # ----------------------------------------------------------
      # --- Remember: The SMA is a floating point number !
      asset.Summe[j] = asset.Summe[j] - asset.SMA[asset.FirstSMA][j] + asset.SMA[asset.LastSMA][j]

   # --------------------------------------------------------------------------------
   asset.sys_DivCount = ( asset.sys_DivCount + 1 ) % asset.sys_Div   # --- Increment LoopCount
   if asset.sys_DivCount == 0 :
      Flow.Add()
      if AvrDebug : Init.SvgUpdate()
      if asset.sys_CsvPath : Csv.Add()

   # ----------------------------------------------------------
   # --- Process Alert ...
   if int( time.time() ) >= Status.StatusTime and Status.AlertList :
      Status.Threshold()
      Status.StatusTime = int( time.time() ) + ( 60 - ( int ( time.time() ) % 60 ) )

   # --------------------------------------------------------------------------------
   asset.FirstSMA = ( asset.FirstSMA + 1 ) % asset.MaxSMA
   # --------------------------------------------------------------------------------

# --------------------------------------- END ---------------------------------------
# -----------------------------------------------------------------------------------
#  sys.exit()
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
