# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#

import sys

import asset
import basics as io
from   Html   import *

import lib
from   lib    import switch, case

# -----------------------------------------------------------------------------------
# --- Global options ...
# -----------------------------------------------------------------------------------
import __main__  as pMain

FuncName = 'fan'

FuncDesc = { 'IconName'     : 'Fan.png',
             'SubName'      : asset.msg['modFanListName'],
             'SubText'      : asset.msg['modFanListText'],
             'iWidth'       : SubMenuWidth,
             'iOnChange'    : f'{asset.PageName}-fan+OnChange',
           }

DutyLow     = '10 15 20 25 30 35 40 45 50 55 60 65 70 75 80 85 90 95 100 105 110 115 120 125 130 135 140 145 150 155 160 165 170 175 180 185 190 195 200 205 210 215 220 225 230 235 240 245 250'
DutyFull    = '50 55 60 65 70 75 80 85 90 95 100 105 110 115 120 125 130 135 140 145 150 155 160 165 170 175 180 185 190 195 200 205 210 215 220 225 230 235 240 245 250 255'

# -----------------------------------------------------------------------------------
# --- Pre-Initialization
# -----------------------------------------------------------------------------------
def pre( site, more, sub=False ) :

   while switch( f'{site}+{more}' ) :
      # ---------------------------------------------------------------------------
      if case( 'fan+save' ):
         # --- Build PWM / ADT "scemd" Value ...
         if asset.cgiInput['FanControlOn'] == "Yes" :
            # ---------------------------------------------------------------------
            jBusy( 'msgCgiWaitBusy', iFlush=True )
            import Scemd
            Fan = Scemd.Fan()

            # ---------------------------------------------------------------------
            try :
               xmlMode = Fan.xmlMode
            except :
               io.touch( asset.f_ScemdError )
               PageRefresh( 'log' )
               sys.exit()

            Mode = asset.cgiInput['FanCurMode']

            try :
               # ---------------------------------------------------------------------
               if xmlMode == "PWM" :
                  # ------------------------------------------------------------------
                  # --- Read "PWM"-Settings ...
                  S = []

                  S.append(  asset.cgiInput['ScemdDutyLow'] )
                  lib.unset( asset.cgiInput,'ScemdDutyLow'  )

                  S.append(  asset.cgiInput['ScemdDutyHigh'] )
                  lib.unset( asset.cgiInput,'ScemdDutyHigh'  )

                  asset.cgiInput['PWM' + Mode + '_S'] = ' '.join( S )

                  for Dev in asset.ScemdDevList :
                     if not Fan.xmlCur['PWM' + Mode + Dev + '_A' ] : continue
                     List = Split( Fan.xmlCur['PWM' + Mode + Dev + '_A'] )
                     C = []
                     for a in range( 0, len( List ) ) :
                        C.append(  asset.cgiInput['ScemdTemp' + Mode + Dev + str(a)] )
                        lib.unset( asset.cgiInput,'ScemdTemp' + Mode + Dev + str(a) )
                     asset.cgiInput['PWM' + Mode + Dev + '_C'] = ' '.join( C )

               # ---------------------------------------------------------------------
               if xmlMode == "ADT" :
                  # ------------------------------------------------------------------
                  # --- Read "ADT"-Settings ...
                  for Dev in asset.ScemdDevList :
                     if not Fan.xmlCur['ADT' + Mode + Dev + '_A' ] : continue
                     List = Split( Fan.xmlCur['ADT' + Mode + Dev + '_A'] )
                     C = []
                     S = []
                     for a in range( 0, len( List ) ) :
                        S.append(  asset.cgiInput['ScemdDuty' + Mode + Dev + str(a)] )
                        lib.unset( asset.cgiInput,'ScemdDuty' + Mode + Dev + str(a) )
                        C.append(  asset.cgiInput['ScemdTemp' + Mode + Dev + str(a)] )
                        lib.unset( asset.cgiInput,'ScemdTemp' + Mode + Dev + str(a) )
                     asset.cgiInput['ADT' + Mode + Dev + '_S'] = ' '.join( S )
                     asset.cgiInput['ADT' + Mode + Dev + '_C'] = ' '.join( C )

               # ------------------------------------------------------------------------
               if xmlMode != 'None' :
                  # ---------------------------------------------------------------------
                  changed = Fan.Update( asset.cgiInput )

                  if not isinstance( changed, (bool) ) :
                     # ---------------------------------------------------------------
                     # --- Update "failed" ...
                     lib.Error( 'CGI: SCEMD exception : {0}'.format( changed ) )
                     io.touch( asset.f_ScemdError )
                     PageRefresh( 'log' )
                     sys.exit()
                  elif changed :
                     # ---------------------------------------------------------------
                     # --- Save Fan-Settings to "scemd.xml" ...
                     changed = Fan.Save()
                     if changed :
                        asset.cgiInput['FanCurrentHash'] = Fan.xmlHash
                     else :
                        io.touch( asset.f_ScemdError )
                        PageRefresh( 'log' )
                        sys.exit()

               # ---------------------------------------------------------------------
            except :
               lib.Error( 'CGI: <b class="red">SCEMD exception :</b> Settings are faulty and could not be read' )
               lib.Error( 'CGI: <b class="red">SCEMD exception :</b> Processing canceled' )
               lib.Error( 'CGI: <b class="red">SCEMD exception :</b> Reinitialization is required' )
               io.ioRm( asset.f_ScemdInit  )
               io.ioRm( asset.f_ScemdError )
               PageRefresh('log' )
               sys.exit()

         # ------------------------------------------------------------------------
         import main
         main.save()
         break

      # ---------------------------------------------------------------------------
      # ---------------------------------------------------------------------------
      if case( 'fan+reset' ):
         # ------------------------------------------------------------------------
         jBusy( 'msgCgiWaitScemdInit', iFlush=True )
         import Scemd
         # ---Reset scemd.xml to default (from APT-File) ...
         Fan = Scemd.Fan( xml=asset.ScemdPatFile )
         if Fan and not Fan.xmlFatal :

            asset.cgiInput = Fan.Reset( reset=True )
            if not asset.cgiInput : return 'log'

            import main
            main.save()
            break
         else :
            return 'log'

      # ---------------------------------------------------------------------------
      # ---------------------------------------------------------------------------
      if case( 'fan+init' ):
         # ------------------------------------------------------------------------
         jBusy( 'msgCgiWaitScemdInit', iFlush=True )
         import Scemd
         # ---FanInit "scemd" values (First-Use) ...
         Fan = Scemd.PatInit( sArchive=asset.cgiInput["SvgArchivePath"], cgi=True )
         if not Fan : return 'log'

         asset.cgiInput = Fan

         import main
         main.save()
         break

      # -----------------------------------------------------------------------------
      break

# -----------------------------------------------------------------------------------
# --- Post-Initialization
# -----------------------------------------------------------------------------------
def post( site, more ) :

   while switch( f'{site}+{more}' ) :
      # -----------------------------------------------------------------------------
      # --- "OnChange" CallBack-Function ...
      if case( 'fan+OnChange' ) :
         break

      # -----------------------------------------------------------------------------
      # --- Return point for "folder" ...
      if case( 'fan+folder' ):
         break

      # ---------------------------------------------------------------------------
      if case( 'fan+path' ):
         # ------------------------------------------------------------------------
         # --- Prepare File-Manager Args .......
         sDict = {   'page'             : 'folder-folder+init',                     # Page to be called up
                     'CalledPageOk'     : f'{asset.PageName}-{FuncName}+folder',    # Calling page "- Ok -"
                     'CalledPageCancel' : f'{asset.PageName}-{FuncName}+folder',    # Calling page "- Cancel -"
                     'FolderBaseTitel'  : asset.msg['cgiTitelBackup'],              # Title / header
                     'FolderMode'       : 'GetFolder',                              # Function inside
                     'FolderHome'       : '',                                       # Select a "Home" folder
                     'FolderCurDir'     : asset.cgiInput['SvgArchivePath'],         # Start directory
                     'FolderCurFile'    : '',                                       # Preselected file
                     'FolderCurExt'     : '*',                                      # File extension (filter)
                     'FileIcon'         : '',                                       # File Icon
                     'FolderRetName'    : 'SvgArchivePath'                          # Return name
                  }

         for key in asset.cgiInput :
            sDict[key] = asset.cgiInput[key]

         # ------------------------------------------------------------------------
         # --- Call File-Manager (Folder-Mode) ...
         CallPage ( sDict )
         sys.exit()
         break

      # -----------------------------------------------------------------------------
      break

   # --------------------------------------------------------------------------------
   # --- Prepare Values ...
   for sKey in asset.cgiInput :
      asset.Out[ sKey] = asset.cgiInput[ sKey ]

# -----------------------------------------------------------------------------------
# --- Function : "engine"
# -----------------------------------------------------------------------------------
def Func ( sSite, sMore ) :

   iWidth   = iMainOutWidth
   iHeight  = iMainHeight

   s           = asset.msg
   Out         = asset.Out

   # --------------------------------------------------------------------------------
   if pMain.Avr[ 'sysExcludeSCEMD' ] == 'Yes' or Out[ 'FanControlOn' ] != 'Yes' :
      PageRefresh( 'Config-expert' )
      sys.exit()

   # --------------------------------------------------------------------------------
   import Scemd
   f  = io.ShareTree( isRoot=True )

   isValid   = Scemd.Validate( asset.Out )
   isInit    = io.check( asset.f_ScemdInit    )
   isError   = io.check( asset.f_ScemdError   )
   isChanged = io.check( asset.f_ScemdChanged )
   isArchive = lib.FindeShare ( Out['SvgArchivePath'] )

   # --------------------------------------------------------------------------------
   asset.Out['AcdActive'] = 1

   for key in asset.cgiInput :
      if 'AcdReturn' in key and asset.cgiInput[ key ] == 'Yes' :
         asset.Out['AcdActive'] = int( key.split( '_', 1 )[-1] )
         break

   # --------------------------------------------------------------------------------
   ContentOpen()
   # --------------------------------------------------------------------------------
   ContentRow ()
   ContentTab ( iWidth, iHeight - 150 )
   # --------------------------------------------------------------------------------

   if not isValid or isError :
   # --------------------------------------------------------------------------------
      Section ( 'ScemdFanInit' )

      RowOpen  ()
      RoBox    ( 'SvgArchivePath'    , 'SvgArchivePath', Column_1b, 219, opt=not isArchive )
      Button   ( 'ButtonSelectFolder', '- Select -', f'{asset.PageName}-{FuncName}+path', iSize=80 )
      RowClose ()

      if isArchive :
      # -----------------------------------------------------------------------------
         if isError :
         # --------------------------------------------------------------------------
            RowOpen  ()
            Label    ( 'ScemdFanInit',      135 )
            Button   ( 'ButtonScemdDefault', '- Reset -', f'{asset.PageName}-{FuncName}+reset', 'ButtonScemdError', iSize=80 )
            RowClose ()
         else :
         # --------------------------------------------------------------------------
            if pMain.Avr[ 'sysVirtualDSM' ] == 'No' :
            # -----------------------------------------------------------------------
               if not bool( asset.SynoInfo ) : asset.SynoInfo = lib.FileConfig( asset.sysSynoConfig )
               ModelName = asset.SynoInfo[ 'upnpmodelname' ]
               if ModelName == 'DDSM' : ModelName = 'VirtualDSM'

               Version = lib.FileConfig( '/etc.defaults/VERSION' )
               patName = 'DSM_{0}_{buildnumber}.pat'.format( ModelName, **Version )

               patFolder = '{0}_{productversion}-{buildnumber}'.format( ModelName, **Version )
               patURL = '{0}/{1}/{2}/{3}'.format    ( asset.ScemdBaseURL, Version['productversion'], Version['buildnumber'], patName )

               sHint = asset.msg['ButtonScemdFanInit'] + ' Dazu wird die DSM-PAT Datei :<br>&nbsp;&nbsp;<b>{0}</b><br>geladen und die <i>"scemd.xml"</i> daraus extrahiert.'.format( patName, patFolder )
               asset.msg['ButtonScemdFanInit'] = sHint

            # -----------------------------------------------------------------------
            RowOpen  ()
            Label    ( 'ScemdFanInit', 135 )
            Button   ( 'ButtonScemdFanInit', '- Init -', f'{asset.PageName}-{FuncName}+init', iSize=80 )
            RowClose ()

            if pMain.Avr[ 'sysVirtualDSM' ] == 'No' :
            # -----------------------------------------------------------------------
               Section  ( 'Info :' )
               RowOpen  ()
               asset.Out['curDSM']  = 'DSM {productversion}-{buildnumber} Update {smallfixnumber}'.format( **Version )
               RoBox    ( 'curDSM'    , 'Version :', Column_1b, 200 )
               RowClose ()

               RowOpen  ()
               asset.Out['curDSM']  = '{0}/{1}/{2}'.format( Out['SvgArchivePath'], patFolder, patName )
               RoBox    ( 'curDSM'    , 'SynoPatFile', Column_1b, 615 )
               RowClose ()

               RowOpen  ()
               asset.Out['curDSM']  = patURL
               RoBox    ( 'curDSM'    , 'Download URL :', Column_1b, 615 )
               RowClose ()
            # -----------------------------------------------------------------------
   else :
   # --------------------------------------------------------------------------------
      try :
      # -----------------------------------------------------------------------------
         if pMain.Avr[ 'sysVirtualDSM' ] == 'Yes' : Section ( 'ScemdControlVm' )
         else                                     : Section ( 'ScemdControlOn' )

         FanMode = Out['FanSysMode']

         RowOpen ()
         # --------------------------------------------------------------------------
         sNice = json.loads( s['SelectModeDict'] )

         for Mode in asset.ScemdModeList :
            try : 
               if not Out[FanMode + Mode + 'disk_C'] : continue
            except :
               continue

            try:
               sSelectRAW += ' ' +        Mode
               sSelectGUI += ' "' + sNice[Mode] + '"'
            except:
               sSelectRAW  =              Mode
               sSelectGUI  =  '"' + sNice[Mode] + '"'

         # --------------------------------------------------------------------------
         if not Out[FanMode + Out['FanCurMode'] + 'disk_C'] : asset.Out['FanSysMode']  = 'LOW'
         SelectGUI ( 'FanCurMode',  'FanCurMode', sSelectGUI, sSelectRAW, 130, 130, True )

         # --------------------------------------------------------------------------
         # --- Get currend Fan-Mode
         # --- fan_config_type_internal="full"     = Full-speed mode
         # --- fan_config_type_internal="high"     = Cool mode
         # --- fan_config_type_internal="low"      = Quiet mode
         # --- fan_config_type_internal="low_stop" = Low-power mode
         if not bool( asset.SynoInfo ) : asset.SynoInfo = lib.FileConfig( asset.sysSynoConfig )
         try    : SynoFan = asset.SynoInfo[ 'fan_config_type_internal' ].upper()
         except : SynoFan = False

         if SynoFan :
            asset.Out['SynoCurFanMode'] = sNice[ SynoFan ]
            RoBox    ( 'SynoCurFanMode', 'SynoCurFanMode', 120, 130, opt=Out['FanCurMode'] != SynoFan )

         RowClose ()

         # --------------------------------------------------------------------------
         Mode = Out['FanCurMode']
         # --------------------------------------------------------------------------
         # --------------------------------------------------------------------------
         if FanMode == "ADT" :
            iCount = 1
            secNice = json.loads( s['modFanDevNice'] )

            sNice = json.loads( asset.msg['AdtActionNice'] )
            AdtHight = int( ( 25.8 * int( Out['FanDivADTcount'] ) ) + 35.0 )

            # -----------------------------------------------------------------------
            for sDev in asset.ScemdDevList :
               if not Out['ADT' + Mode + sDev + '_C'] : continue
               # --------------------------------------------------------------------
               C = Out['ADT' + Mode + sDev + '_C'].split(' ')
               S = Out['ADT' + Mode + sDev + '_S'].split(' ')
               A = Split ( Out['ADT' + Mode + sDev + '_A'] )

               SectionOpen( secNice[sDev], iIndex=iCount )
               iCount += 1

               for a in range( 0, len(A) ) :
                  sName = Mode + sDev + str(a)
                  asset.Out['ScemdDuty'  + sName] = S[a]
                  asset.Out['ScemdTemp'  + sName] = C[a]

                  if A[a] in sNice : sID = sNice[ A[a] ]
                  else             : sID =        A[a]

                  asset.Out['ScemdLimit' + sName] = sID
                  RowOpen  ()
                  Select   ( 'ScemdTemp' + sName, asset.Out['ScemdLimit' + sName], asset.AdtTempList,  Column_1, 55 )
                  Label    ( '°C', 20 )

                  Select   ( 'ScemdDuty' + sName, '&nbsp;:',                       asset.AdtSpeedList, 10, 55 )
                  Label    ( '%', 20 )
                  RowClose ()

               SectionClose()
         # --------------------------------------------------------------------------
         # --------------------------------------------------------------------------
         if FanMode == "PWM" :
            iWidth   = 250
            iGap     =   5

            sNice = json.loads( asset.msg['PwmActionNice'] )
            PwmHight = int( ( 25.8 * int( Out['FanDivPWMcount'] ) ) + 35.0 )

            # -----------------------------------------------------------------------
            SectionOpen( 'ScemdDutyPwm', iIndex=1 )

            S = Out['PWM' + Mode + '_S' ].split(' ')
            asset.Out['ScemdDutyLow']  = S[0]
            asset.Out['ScemdDutyHigh'] = S[1]

            RowOpen  ()
            Select   ( 'ScemdDutyLow' , 'Normal :', DutyLow,  Column_1a, 55 )
            RowClose ()

            RowOpen  ()
            Select   ( 'ScemdDutyHigh', 'Full Speed :', DutyFull, Column_1a, 55 )
            RowClose ()

            SectionClose()
            # -----------------------------------------------------------------------
            # -----------------------------------------------------------------------
            iCount = 2
            for sDev in asset.ScemdDevList :
               if not Out['PWM' + Mode + sDev + '_A'] : continue

               # --------------------------------------------------------------------
               C = Out['PWM' + Mode + sDev + '_C'].split(' ')
               A = Split ( Out['PWM' + Mode + sDev + '_A'] )

               secNice = json.loads( s['modFanDevNice'] )

               SectionOpen( secNice[sDev], iIndex=iCount )
               iCount += 1

               for a in range( 0, len(A) ) :
                  sName = Mode + sDev + str(a)
                  asset.Out['ScemdTemp'  + sName] = C[a]

                  if A[a] in sNice : sID = sNice[ A[a] ]
                  else             : sID =        A[a]

                  RowOpen  ()
                  Label    ( sID, Column_1a )
                  Select   ( 'ScemdTemp' + sName , '@',  asset.PwmTempList,  10, 55 )
                  Label    ( '°C', 20 )
                  RowClose ()

               SectionClose()
      # -----------------------------------------------------------------------------
      except :
         lib.Error( 'CGI: SCEMD exception : Settings are faulty and could not be read' )
         lib.Error( 'CGI: SCEMD exception : Processing canceled' )
         lib.Error( 'CGI: SCEMD exception : Reinitialization is required' )
         io.ioRm( asset.f_ScemdInit  )
         io.ioRm( asset.f_ScemdError )
         return 'Fan'
         sys.exit()

   # --------------------------------------------------------------------------------
   if not isError and isValid :
      Section ( 'ScemdConfirm' )

      RowOpen  ()
      Label    ( 'ScemdConfirm',   Column_1 )

      if isChanged : 
         Button ( 'ButtonScemdDefault', '- ReInit -', f'{asset.PageName}-{FuncName}+init',  'ButtonScemdReInit', iSize=80 )

      Button (    'ButtonScemdDefault', '- Reset -',  f'{asset.PageName}-{FuncName}+reset', iSize=80 )
      RowClose ()

      # -----------------------------------------------------------------------------
      ContentTabEnd()
      ContentRowEnd()

      # -----------------------------------------------------------------------------
      # --- Display "Save" & "Undo" Button ...
      SaveUndo( asset.PageName, FuncName, iSize=80, iWidth=iMainOutWidth )
   else :
      ContentTabEnd()
      ContentRowEnd()

   # --------------------------------------------------------------------------------
   Hide ( 'SvgArchivePath'    )
   Hide ( 'FanControlOn'      )
   Hide ( 'FanProtocolOn'     )
   Hide ( 'FanConfirmationOn' )

   # --------------------------------------------------------------------------------
   ContentClose()


# --------------------------------------- END ---------------------------------------
# -----------------------------------------------------------------------------------
#  sys.exit()
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
