# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# coding: utf-8                                                                      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#

# -----------------------------------------------------------------------------------
# --- Global options :
# -----------------------------------------------------------------------------------
Modules = '/usr/syno/synoman/webman/modules'
Groups  = '/etc/group'  # -->> DSM 7 as admin readable
User    = '/etc/passwd' # -->> DSM 7 as admin readable
Debug   = False

# -----------------------------------------------------------------------------------
import sys

# -----------------------------------------------------------------------------------
import os
import re
import json
import time
from   http import cookies
import subprocess

from   traceback import format_exc as Trace

import asset

# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
class BaseException( Exception ) :
   """Base exception for this Module"""
   # --- "Exception" is a Built-in Exceptions

# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
class mainException( BaseException ) :
   """Main Exception handler"""

# -----------------------------------------------------------------------------------
# --- Helper function for getting the current exception in a try/except block
# -----------------------------------------------------------------------------------
def getException() :
   return sys.exc_info()[1]

# -----------------------------------------------------------------------------------
# Init "SynoToken" and 'synoCheck.json' :
# -----------------------------------------------------------------------------------
def check( AppName, iLap = 5 ):

   iErr     = False
   sLog     =''
   sStamp   = time.strftime('%Y.%m.%d %H:%M:%S')

   iLapTime = time.time()

   # --------------------------------------------------------------------------------
   # --- Check "TimeStamp" of 'synoCheck.json' ...
   # --------------------------------------------------------------------------------
   try :
      sTime = int( os.path.getmtime( asset.cgiSynoCheck ) )
      if sTime and int( time.time() ) - sTime <= iLap :
         if Debug : print ( f'{sStamp} check: Timestamp of [synoCheck] is still valid', file=sys.stderr )
         # --- Renew 'TimeStamp' :
         os.system( 'touch ' + asset.cgiSynoCheck )
         return
   except :
      if Debug : print ( f'{sStamp} check: Query of [synoCheck] failed', file=sys.stderr )
      pass


   try :
      # -----------------------------------------------------------------------------
      # --- TimeStamp has expired ...
      SynoToken = getSynoToken()
      # --- Without SynoToken ... delete 'synoCheck.json' and return ...
      if not SynoToken :
         print ( f'{sStamp} check: Query of [SynoToken] failed', file=sys.stderr )

         try    : os.remove( asset.cgiSynoCheck )
         except : pass

         return

      # -----------------------------------------------------------------------------
      # --- Init 'synoCheck.json' ... (read previous / set default)
      if os.path.exists( asset.cgiSynoCheck ) :
         Cookies = json.loads( open( asset.cgiSynoCheck ).read().strip() )
         if Debug : print ( f'{sStamp} check: [synoCheck] read in successfully', file=sys.stderr )
      else :
         Cookies = { 'SynoToken' : SynoToken, 'SynoUser' : '', 'SynoAdmin' : None, 'appPrivileg' : None, 'SynoID' : '' }

      # -----------------------------------------------------------------------------
      if SynoToken != Cookies['SynoToken'] or not Cookies['SynoUser'] :
         if Debug and SynoToken != Cookies['SynoToken'] : print ( f'{sStamp} check: Current [SynoToken] does not match', file=sys.stderr )

         Cookies = { 'SynoToken' : SynoToken, 'SynoUser' : '', 'SynoAdmin' : None, 'appPrivileg' : None, 'SynoID' : '' }
         if Debug : print ( f'{sStamp} check: Continue with default values for [synoCheck]', file=sys.stderr )

         SynoUser = getAuthentification( SynoToken )
         if not SynoUser :
            print ( f'{sStamp} check: Query of [User] failed', file=sys.stderr )

            try    : os.remove( asset.cgiSynoCheck )
            except : pass

            return

         Cookies['SynoUser'] = SynoUser
         if Debug : print ( f'{sStamp} check: Currently logged in user = [{SynoUser}]', file=sys.stderr )

      # -----------------------------------------------------------------------------
      # --- Renew 'Admin'-Status ...
      if Cookies['SynoAdmin'] is None :
         Cookies['SynoAdmin'] = CheckUserIsAdmin( SynoUser )
         if Debug : print ( f'{sStamp} check: User [{SynoUser}] is admin : {Cookies["SynoAdmin"]}', file=sys.stderr )

      # -----------------------------------------------------------------------------
      # --- Renew 'App'-Privileg ...
      if Cookies['appPrivileg'] is None :
         Cookies['appPrivileg'] = CheckAppPrivileg( SynoUser, f'SYNO.SDS.{AppName}.Application' )
         if Debug : print ( f'{sStamp} check: User [{SynoUser}] has app (SYNO.SDS.{AppName}.Application) privileg : {Cookies["appPrivileg"]}', file=sys.stderr )

      if not Cookies['appPrivileg'] :
         print ( f'{sStamp} check: The user [{SynoUser}] does not have permission for [AvrLogger]', file=sys.stderr )

         try    : os.remove( asset.cgiSynoCheck )
         except : pass

         return

      # -----------------------------------------------------------------------------
      # --- Renew Session-ID ...
      try    : Cookies['SynoID'] = parseCOOKIE( 'id' )
      except : Cookies['SynoID'] = False

      if not Cookies['SynoID'] :
         print ( f'{sStamp} check: Query of [Session ID] failed', file=sys.stderr )

         try    : os.remove( asset.cgiSynoCheck )
         except : pass

         return

      # -----------------------------------------------------------------------------
      with open( asset.cgiSynoCheck, 'w+', encoding='utf-8' ) as f:
         f.write( json.dumps( Cookies, sort_keys=True, indent=4 ) )

      if Debug :
         print ( f'{sStamp} check: [synoCheck] successfully renewed and saved :', file=sys.stderr )
         print ( json.dumps( Cookies, sort_keys=True, indent=4 ), file=sys.stderr )

      # -----------------------------------------------------------------------------
      if asset.f_Dsm7 :
         import shutil
         try    : shutil.chown ( asset.cgiSynoCheck, user=asset.SpkName, group=asset.SpkName )
         except : pass

   except ( BaseException ) :
      # -----------------------------------------------------------------------------
      print ( f'{sStamp} BaseException in : {getException()}', file=sys.stderr )
      return False

   except Exception :
      # -----------------------------------------------------------------------------
      print ( f'{sStamp} Exception :\n{Trace()}', file=sys.stderr )
      return False

   # --------------------------------------------------------------------------------
   if Debug : print( f'{sStamp} check: Done in %.6fs' % ( time.time() - iLapTime ), file=sys.stderr )

# -----------------------------------------------------------------------------------
# --- Perform an "SynoToken" request to prevent a CSRF.
# -----------------------------------------------------------------------------------
def getSynoToken():

   # -->> DSM 7 need "root"
   # --- '/usr/syno/synoman/webman/modules/login.cgi'

   try :
      TOKEN, ERR = subprocess.Popen( Modules + '/login.cgi', shell=True, stdout=subprocess.PIPE, stdin=subprocess.PIPE ).communicate()
      TOKEN = TOKEN.decode( 'utf-8' ).replace(  '\n', '' )
      TOKEN = json.loads( '{ %s }' % re.search( '%s(.*)%s' % ( '{', '}' ), TOKEN ).group( 1 ) )
      return TOKEN.get( 'SynoToken', False )
   except :
      return False

# -----------------------------------------------------------------------------------
# --- Perform an "authentication" check to ensure only logged-in users can access the page.
# --- Will output the user name if the user has logged in.
# -----------------------------------------------------------------------------------
def getAuthentification( sToken ):

   # -->> DSM 7 as admin readable
   # --- '/usr/syno/synoman/webman/modules/authenticate.cgi'
   try :
      QUERY = os.environ['QUERY_STRING']
      os.environ['QUERY_STRING'] = 'SynoToken=' + sToken
      AUTH, ERR = subprocess.Popen( Modules + '/authenticate.cgi', shell=True, stdout=subprocess.PIPE, stdin=subprocess.PIPE ).communicate()
      AUTH = AUTH.decode( 'utf-8' ).strip()
      os.environ['QUERY_STRING'] = QUERY
      return AUTH
   except :
      return False

# -----------------------------------------------------------------------------------
# --- Check if User is Admin :
# -----------------------------------------------------------------------------------
def CheckUserIsAdmin( sUser ) :

   try :
      with open( Groups, 'r', encoding='utf-8' ) as f:
         iGroups = f.read().strip().split( '\n' )

      sAdmins = [ sLine for sLine in iGroups if "administrators" in sLine ][0]
   except :
      return False

   iMembers = sAdmins.split( ':' )[-1].split( ',' )
   if sUser in iMembers : return True
   else                 : return False

# -----------------------------------------------------------------------------------
# --- Check App Privileg :
# -----------------------------------------------------------------------------------
def CheckAppPrivileg( sUser, sApp ) :

   # -->> DSM 7 as admin readable
   # --- '/usr/syno/synoman/webman/modules/authenticate.cgi'

   try :
      Data = json.loads( open( f'/usr/syno/etc/preference/{sUser}/usersettings' ).read().strip() )
      if sApp in Data[ 'Desktop' ][ 'valid_appview_order' ] :
         return True
      else :
         return False
   except :
      return False

# -----------------------------------------------------------------------------------
# --- Parse (get) 'HTTP_COOKIE' Cookie's ...
# -----------------------------------------------------------------------------------
def parseCOOKIE ( sCookie ) :

   try :
      httpCookie = cookies.SimpleCookie()
      httpCookie.load( os.environ.get( 'HTTP_COOKIE' ) )
      return httpCookie[ sCookie ].value
   except :
      return False

# -----------------------------------------------------------------------------------
# --- Quick check Session-ID :
# -----------------------------------------------------------------------------------
def quick( iLap = 600 ) :

   sStamp = time.strftime('%Y.%m.%d %H:%M:%S')

   try :
      # -----------------------------------------------------------------------------
      # --- The timestamp of 'synoCheck.json' is only valid for 10 minutes ...
      sTime = int( os.path.getmtime( asset.cgiSynoCheck ) )
      if sTime and int( time.time() ) - sTime > iLap :
         if Debug : print ( f'{sStamp} quick: Timestamp has expired', file=sys.stderr )
         return False
   except :
      if Debug : print ( f'{sStamp} quick: Query of [synoCheck] failed', file=sys.stderr )
      return False

   try :
      # -----------------------------------------------------------------------------
      # --- Get out of here without 'HTTP_COOKIE's ...
      if not 'HTTP_COOKIE' in os.environ or not os.path.exists( asset.cgiSynoCheck ) :
         if Debug : raise mainException( f'{sStamp} quick: Query of [HTTP_COOKIE] failed' )
         return False

      # -----------------------------------------------------------------------------
      # --- Get the Session ID from 'HTTP_COOKIE's ...
      sessionID = parseCOOKIE( 'id' )
      if not sessionID :
         if Debug : raise mainException( f'{sStamp} quick: Query of [Session ID] failed' )
         return False

      # -----------------------------------------------------------------------------
      # --- Compare the current Session ID with the saved one ...
      if sessionID != json.loads( open( asset.cgiSynoCheck ).read().strip() ).get( 'SynoID', False )  :
         if Debug : raise mainException( f'{sStamp} quick: Current [Session ID] does not match' )
         return False

   except ( BaseException ) :
      # -----------------------------------------------------------------------------
      print ( f'{sStamp} BaseException in : {getException()}', file=sys.stderr )
      return False

   except Exception :
      # -----------------------------------------------------------------------------
      print ( f'{sStamp} Exception :\n{Trace()}', file=sys.stderr )
      return False

   # --------------------------------------------------------------------------------
   if Debug : print ( f'{sStamp} quick: Quick test successfully completed', file=sys.stderr )
   return True

# -----------------------------------------------------------------------------------
# --- Check Schedule-Status ...
# --- 0 = Scheduling is "OFF" / -1 = Forbidden time / 1 = Allowed time
# --- If necessary: start / stop AvrLogger.
# -----------------------------------------------------------------------------------
def schedule() :

   import basics as io

   from   lib  import switch, case
   import lib
   import Cron

   import __main__ as pMain

   Cron.checkTime()
   ChangeOrder   = False

   if pMain.AvrPid and os.path.exists( asset.f_AvrDisabled ) :
      os.system( ' '.join(( asset.script, 'cgi', 'stop' )) )
      ChangeOrder = True
      pMain.AvrPid = []

   while switch( str( Cron.ScheduleStatus ) ) :
      if case( '-1' ):
         # --- Scheduling is "ON" -1 = Forbidden time
         if pMain.AvrPid :
            # --- Stop Daemon / enter Hibernation-Mode
            io.touch( asset.f_CgiUpdate )
            os.system( ' '.join(( asset.CronScript, 'stop', '&' )) )
            ChangeOrder = True
            pMain.AvrPid = []
         break

      if case( '0' ):
         # --- Scheduling is "OFF"
         if not pMain.AvrPid and not os.path.exists( asset.f_AvrDisabled ) :
            # --- If Daemon is enabled and not running ...
            if os.path.exists( asset.avrPidFile ) :
               Print( time.strftime('%Y.%m.%d %H:%M:%S ') + "GUI: AvrLogger quit unexpectedly. Restart initiated ..." )

            io.touch( asset.f_CgiUpdate )
            # --- Start Daemon ...
            os.system( ' '.join(( asset.script, 'cgi', 'start', '&' )) )
            ChangeOrder = True
            pMain.AvrPid = GetPPidOf( f'{asset.SpkDaemon}.py' )
         break

      if case( '1' ):
         # --- Scheduling is "ON“ 1 = Allowed time
         if not pMain.AvrPid and not os.path.exists( asset.f_AvrDisabled ) :
            # --- Start Daemon / exit Hibernation-Mode
            io.touch( asset.f_CgiUpdate )
            os.system( ' '.join(( asset.CronScript, 'start', '&' )) )
            ChangeOrder = True
            pMain.AvrPid = GetPPidOf( f'{asset.SpkDaemon}.py' )
         break

      break

   # --------------------------------------------------------------------------------
   if os.path.exists( asset.f_AvrDisabled ):
      iID = 0
   else:
      if pMain.AvrPid : iID = 1
      else            : iID = 2

   sIconNice = json.loads( asset.msg['nacCgiStausIcn'] )
   sIcon     = asset.cgiNavIcon[ iID ]
   sText     = sIconNice[ str( iID ) ]
   io.ioWrite( json.dumps( { 'rtfStatusIcon' : sIcon, 'rtfStatusText' : sText }, sort_keys=True, indent=3 ), asset.f_avrStatus )

   return ChangeOrder

# -----------------------------------------------------------------------------------
# --- Get PID(s) by Name :
# --- Examine every ''cmdline'' to see if ''Name'' is included.
# -----------------------------------------------------------------------------------
def GetPPidOf( sName, opt=False ) :

   PID = []
   for sDir in os.listdir( '/proc' ) :
      if sDir.isdigit() :
         try:
            if sName in open( f'/proc/{sDir}/cmdline' ).read() :
               PID.append( int( sDir ) )
         except:
            continue

   return PID

# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
def Print ( *args, **kwargs ) :

   print( *args, file=sys.stderr, **kwargs )

# --------------------------------------- END ---------------------------------------
# -----------------------------------------------------------------------------------
#  sys.exit()
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
