# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# coding: utf-8                                                                      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#
import os
import sys
import time
from   shutil  import copy, chown

import asset
import basics   as io
from   lib      import switch, case
import lib

import Sys
import Csv
import Ookla
import Status
import Memory
import Sensors
import Snmp
import Flow
import Oid
import Cron
import Push
import Smart

import Flow
import Svg

# -----------------------------------------------------------------------------------
# --- Global options ...
# -----------------------------------------------------------------------------------
import __main__ as avr

stat  = Status

# -----------------------------------------------------------------------------------
rel = sys.modules[ __name__ ]

rel.relBaseList         = None
rel.relTimeBase         = None
rel.relEnvironmental    = None
rel.relSvgCopy          = None
rel.relSvgArchiving     = None
rel.relSensors          = None

# --------------------------------------------------------------------------
# --- "Main"-Initialization:
# --- Konfig-Datei einlesen - Werte validieren ...
# --- Array's und Funktionen inizialisieren ...
# --------------------------------------------------------------------------
def init( BootInit=True, smallInit=False, AvrDebug=False ):

   lib.TimerStart()
#  if smallInit : AvrDebug = True
#  else         : AvrDebug = avr.AvrDebug
   if smallInit : AvrDebug = smallInit

   # -----------------------------------------------------------------------
   # --- "Always" load the config first ...
   lib.TimerStart()
   asset.AvrConf  = lib.ConfigRead( asset.avrConfigFile )
   asset.SynoInfo = lib.FileConfig( asset.sysSynoConfig )
   lib.TimerStop()

   # -----------------------------------------------------------------------
   if BootInit :
      lib.Echo ( 0, asset.msg[ 'msgWellcome'   ] )
      lib.Echo ( 1, asset.msg[ 'msgConfigRead' ] )
      lib.TimerStart()
      Memory.init()
      lib.TimerStop()
      lib.Echo ( 1, "| SYS: System-RAM = " + str(asset.sys_RamSizeGB) + " GB (" + str(asset.sys_RamSizeKB) + " kB)" )
   else :
      lib.Echo ( 0, asset.msg[ 'msgConfigUpdate' ] )
   # -----------------------------------------------------------------------

   # -----------------------------------------------------------------------
   if BootInit :
      lib.TimerStart()
      f = Smart.Smart( Debug=avr.AvrSmart )
      asset.sysStorage = f.getSynoStorage()
      lib.TimerStop()

      lib.Echo ( 1, '| SYS: Storage pool has been initialized.' )

   # ----------------------------------------------------------
   lib.TimerStart()
   Snmp.init( BootInit )
   lib.TimerStop()
   lib.Echo ( 1, asset.msg['msgEnvSensor'] )

   lib.TimerStart()
   Ookla.init()
   lib.TimerStop()
   lib.Echo ( 1, asset.msg['msgOoklaInit'] )

   # ----------------------------------------------------------
   if not rel.relBaseList :
      lib.TimerStart()
      Sensors.init()
      Sys.GetSystemInfo()
      lib.TimerStop()
      lib.Echo ( 1, asset.msg['msgSystemSensors'] )
   # ----------------------------------------------------------

   # ----------------------------------------------------------
   Sys.SetAdminNotification()
   Sensors.Nut()  # Determine UPS support
   Sensors.initUps()
   Csv.init()
   Status.init()
   BaseList ()
   MinMax   ()
   DataBase ()

   # ----------------------------------------------------------
   if smallInit : return

   # ----------------------------------------------------------
   # --- Only if this is a "restart" of the daemon ...
   if BootInit and not AvrDebug and not os.path.exists( asset.f_RestoreConf ) :
      if asset.AvrConf["LogClearOn"]    == "Yes" :
         # --- Clear DB with rotate ...
         Sys.RotateClear( asset.avrLogDataFile )

      if asset.AvrConf["LogRealtimeOn"] == "Yes" :
         # --- Add the expected number of values ...
         Sys.AddOffline()

      io.ioRm( asset.f_RestoreConf )

   # ----------------------------------------------------------
   # --- Must be placed after "AddOffline", otherwise TimeStamp will be falsified!
   lib.TimerStart()
   Sys.TrimFile  ( 2880, asset.avrLogDataFile )
   lib.TimerStop ()
   if BootInit : lib.Echo ( 1, asset.msg['msgDatabaseSize'] )
   # ----------------------------------------------------------

   # ----------------------------------------------------------
   if not rel.relBaseList :
      lib.TimerStart    ()
      Sensors.FirstOid  ()
      Sensors.FirstLoad ()
      Sensors.FirstDisk ()
      Sensors.FirstEth  ()
      FirstUse          ()
      lib.TimerStop     ()
      lib.Echo ( 1, asset.msg['msgFirstUseSMA'] )
   # ----------------------------------------------------------

   # ----------------------------------------------------------
   lib.TimerStart()
   Cron.init     ()                          # --- Init CronJob Values
   Cron.checkTime()                          # --- Init Schedule Status
   Cron.CronJob  ()  # $crn_ScheduleStatus   # --- Init Cron Job(s)
   lib.TimerStop ()
   if BootInit  : lib.Echo ( 1, asset.msg['msgCronJobDone'] )
   # ----------------------------------------------------------

   InitSvgArchiving()
   InitSvgCopy()

   # ----------------------------------------------------------
   lib.TimerStart()
   Svg.Init()
   Svg.Prio()
   lib.TimerStop ()
   lib.Echo ( 1, asset.msg['msgSvgPrioInit'] )
   # ----------------------------------------------------------

   lib.TimerStart()
   # ----------------------------------------------------------
   Flow.ChangeMin()

   if not rel.relBaseList :
      Flow.init  ()

   lib.TimerStop ()
   lib.Echo ( 1, asset.msg['msgFlowInit'] )
   # ----------------------------------------------------------
   if not rel.relBaseList :
      lib.TimerStart()
      Svg.PlotInit  ()
      Svg.SVG       ()
      lib.TimerStop ()
      lib.Echo ( 1, asset.msg['msgSvgPlotInit'] )
   else :
      SvgUpdate()
   # ----------------------------------------------------------

   # ----------------------------------------------------------
   if BootInit and not asset.AvrConf["StatusPushRegistered"] :
      Push.ReInit ()
   # ----------------------------------------------------------

   # ----------------------------------------------------------
   # --- After successful "initialization" save the config
   asset.AvrConf["cgiLastCommand"] = ""
   lib.TimerStart()
   lib.ConfigSave()
   lib.TimerStop ()
   if BootInit : lib.Echo ( 1, asset.msg['msgConfigSaved'] )
   # ----------------------------------------------------------

   # ----------------------------------------------------------
   lib.TimerStop()
   if BootInit : lib.Echo ( 1, asset.msg['msgInitEnd'] )
   else :        lib.Echo ( 1, asset.msg['msgConfigUpdateEnd'] )
   # ----------------------------------------------------------

   # ----------------------------------------------------------
   if BootInit :
      lib.Echo ( 0, asset.msg['msgSeparator'] )
      lib.Echo ( 2, 'Tick-Time : %2.3f Prescaler : %3d  DataBase : %4d  SMA depth : %3d' % ( asset.sys_Tick, asset.sys_Div, asset.sys_MaxLines, asset.SMAdepth) )
      lib.Echo ( 2, 'TimeBase  : %5s SVG-Tick  : %3s' % ( asset.AvrConf["LogBase"], asset.AvrConf["SvgTick"] ) )       
      # ----------------------------------------------------------
      lib.Echo ( 0, asset.msg['msgSeparator'] )

      for Axis in range( 0, 6 ) :
         if not asset.ScalingAxis_Names[Axis] : continue
         printMIN = asset.ScalingAxis_Min[Axis]
         printMAX = asset.ScalingAxis_Max[Axis]
         msg = 'Axis #%1d :%5d - %-5d' % ( Axis, printMIN, printMAX )
         for j in asset.ScalingAxis_Names[Axis] :
            msg += ' [%2d]' % (j)

         lib.Echo ( 2, msg )

   lib.Echo ( 0, asset.msg['msgSeparator'] )
   # ----------------------------------------------------------

   io.ioRm( asset.f_CgiUpdate )
   io.ioRm( asset.f_AvrLock )

   # --- Set AvrTick-Time (Daemon) ...
   asset.LoopTime = time.time() + asset.sys_Tick

# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
# --- Init (Update) Config
# --- Trap "10" Handle -> Signal from GUI (.cgi) : Config changed ...
# -----------------------------------------------------------------------------------
def ConfigUpdate( signum, frame ) :

   init( False )

# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
# --- Update SVG-Plot :
# --- Trap "16" Handle -> Signal from cronjob - SvgTick has expired, update plot
# -----------------------------------------------------------------------------------
def SvgUpdate( signum=False, frame=None ) :

   io.touch( asset.f_CrnFlag )
   lib.TimerStart()

   ReScan = False
   OldMax = [None] * 6
   flow   = Flow
   # --------------------------------------------------------------------------------
   # --- Walk through AxisList (Axis 0 - 5) and get Max-Value ...
   io.touch( asset.f_Heartbeat )

   for Axis in range( 0, 6 ) :
      # -----------------------------------------------------------------------------
      # --- Copy current Max-Value to distinguish "Up-Scale" or "Down-Scale".
      OldMax[ Axis ] = asset.ScalingAxis_Max[ Axis ]
      # -----------------------------------------------------------------------------
      # --- Walk through all Axis-Values and get Max-Value,
      # --- Round value depending on its size ...
      Flow.RoundUp( Axis, max( row[ Axis ] for row in flow.maxAxis ) )

      try:    CmpMinMax += f' {asset.ScalingAxis_Min[ Axis ]} {asset.ScalingAxis_Max[ Axis ]}'
      except: CmpMinMax  =  f'{asset.ScalingAxis_Min[ Axis ]} {asset.ScalingAxis_Max[ Axis ]}'
      # -----------------------------------------------------------------------------
      # --- If the current Max-Value is "greater than" the previous one ...
      # -----------------------------------------------------------------------------
      io.touch( asset.f_Heartbeat )

   # --------------------------------------------------------------------------------
   if CmpMinMax != asset.ScalingMinMax :
      ReScan = True
      io.ioWrite( CmpMinMax + " :svgUpdate\n", asset.avrScalingFile )
      asset.ScalingMinMax = CmpMinMax

   # --------------------------------------------------------------------------------
   # --- Update the Plot's ...
   Svg.PlotInit()
   Svg.SVG()

   lib.TimerStop()

   # --------------------------------------------------------------------------------
   # --- This part is cosmetic only and purely informative ...
   if not signum:
      lib.Echo( 1, asset.msg['msgSvgPlotUpdate'] )

   if not signum and ReScan :
      lib.Echo( 0, asset.msg['msgSeparator'] )
      for Axis in range( 0, 6 ) :
         if not asset.ScalingAxis_Names[Axis] : continue

         printOld = OldMax[Axis]
         printMIN = asset.ScalingAxis_Min[Axis]
         printMAX = asset.ScalingAxis_Max[Axis]

         printCmp = "EQ"
         if asset.ScalingAxis_Max[Axis] != OldMax[Axis] :
            if asset.ScalingAxis_Max[Axis] < OldMax[Axis] :
               printCmp = "LT"
            else :
               printCmp = "GT"

         msg = 'Axis #%1d :%5d - %-5d >> %s %-5d' % ( Axis, printMIN, printMAX, printCmp, printOld )
      
         for j in asset.ScalingAxis_Names[Axis] :
            msg += ' [%2d]' % (j)

         lib.Echo( 2, msg )
      lib.Echo( 0, asset.msg['msgSeparator'] )

   # --------------------------------------------------------------------------------
   io.ioRm( asset.f_CrnFlag )

# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
# --- Check Timer and archive SVG graphic
# --- Trap "30" Handle -> Signal from cronjob - Archive timer has expired, archive plot
# -----------------------------------------------------------------------------------
def SvgArchive( signum, frame ) :

   # --- Only if there is something to archive ...
   if not io.check( asset.SvgPlot ) or asset.AvrConf["SvgArchiveMode"] == "Off" :
      # --- Clear CronLock locking ...
      io.ioRm( asset.f_CrnFlag )
      return

   # ----------------------------------------------------------
   # --- Prüfen ob es sich um einen gültigen Pfad handelt ...
   arr = lib.FindeShare ( asset.AvrConf["SvgArchiveFolder"] )

   if arr :
      SvgPath = os.path.join( *arr )
   else :
      asset.AvrConf["SvgArchiveMode"] = "Off"
      io.ioRm( asset.f_CrnFlag )
      if not asset.AvrConf["SvgArchiveFolder"] : return
      lib.Error( asset.msg['errArchivPath'] + ' "' + asset.AvrConf["SvgArchiveFolder"] + '"' )
      return

   TimeOut = int( time.time() ) + 10
   # ----------------------------------------------------------
   # --- Wait for valid Plot or Timeout ...
   while io.check( asset.f_SvgLock ) :
      if int( time.time() ) >= TimeOut :
         lib.Error( asset.msg['errArchivTimeout'] )
         # --- Clear Daemon (ArchiveLock) locking
         io.ioRm( asset.f_CrnFlag )
         return

      time.sleep( 0.005 )

   # ----------------------------------------------------------
   if not avr.AvrDebug :
      SvgFile = '/'.join(( SvgPath, time.strftime('%Y.%m.%d-%H.%M.svg' ) ))
      copy( asset.SvgPlot, SvgFile )
      chown( SvgFile, user='admin', group='users' )
      os.chmod( SvgFile, 0o755 )

   # --- Clear Daemon (ArchiveLock) locking
   io.ioRm( asset.f_CrnFlag )

# --------------------------------------------------------------------------
# --- Prepare SVG-Copy ...
# --------------------------------------------------------------------------
def InitSvgCopy() :

   if asset.AvrConf["SvgCopyOn"] != "Yes" : return

   # -----------------------------------------------------------------------
   if rel.relSvgCopy and rel.relSvgCopy == ''.join(( asset.AvrConf["SvgCopyOn"], asset.AvrConf["SvgCopyPath"] )) :
      return

   SvgPath = os.path.dirname ( asset.AvrConf['SvgCopyPath'] )
   SvgName = os.path.basename( asset.AvrConf['SvgCopyPath'] )

   arr = lib.FindeShare ( SvgPath )

   if arr :
      asset.AvrConf["SvgCopyPath"] = os.path.join( arr[1], SvgName )
   else :
      asset.AvrConf["SvgCopyOn"] = "No"
      lib.Error( asset.msg['errSvgCopyPath'] + ' "' + asset.AvrConf["SvgCopyPath"] + '"' )
      return

   # -----------------------------------------------------------------------
   rel.relSvgCopy = ''.join(( asset.AvrConf["SvgCopyOn"], asset.AvrConf["SvgCopyPath"] ))

# --------------------------------------------------------------------------
# --- Prepare SVG-Archiving (Archive Mode) ...
# --------------------------------------------------------------------------
def InitSvgArchiving() :

   if asset.AvrConf["SvgArchiveMode"] == "Off" : return

   # -----------------------------------------------------------------------
   if rel.relSvgArchiving == ''.join(( asset.AvrConf["SvgArchiveMode"], asset.AvrConf["SvgArchiveFolder"] )) :
      return

   arr = lib.FindeShare( asset.AvrConf["SvgArchiveFolder"] )
   
   if arr :
      asset.AvrConf["SvgArchiveFolder"] = arr[1]
   else :
      asset.AvrConf["SvgArchiveMode"] = "Off"
      return

   # -----------------------------------------------------------------------
   rel.relSvgArchiving = ''.join(( asset.AvrConf["SvgArchiveMode"], asset.AvrConf["SvgArchiveFolder"] ))

# --------------------------------------------------------------------------
# --- Update DataBase :
# --- Extend the number of values per line to the expected number
# --- Change number format from integer to floating point
# --------------------------------------------------------------------------
def UpdateDB ( Float, TabLast ) :

   Sys.RotateClear( asset.avrLogDataFile )

   sAdd = []

   if len(asset.DB[0].split()) < TabLast :
      lib.Error ( asset.msg['errDatabaseSize1'] + ' \"' + str(TabLast) + '\"' )
      sAdd = [ 0.0 ] * ( TabLast - len(asset.DB[0].split()) )

   if not Float :
      lib.Error ( asset.msg['errDatabaseSize2'] )

   # -----------------------------------------------------------------------
   for iID in range( 0, len(asset.DB) ) :

      # --------------------------------------------------------------------
      if Float  : iLine = [        float ( x )                      for x in asset.DB[iID].split() ]
      else      : iLine = [ round( float ( x ) / 10,asset.rnd_SMA ) for x in asset.DB[iID].split() ]

      iLine.extend( sAdd )
      asset.DB[iID] = ' '.join( '%.3f' % ( abs( x ) ) for x in iLine )

   # -----------------------------------------------------------------------
   io.ioWrite( '\n'.join( asset.DB ) + '\n', asset.avrLogDataFile )

   # -----------------------------------------------------------------------
   lib.Error ( asset.msg['errDatabaseSize3'] )

# --------------------------------------------------------------------------
# --- Init Deamon "First Use"  ...
# --------------------------------------------------------------------------
def FirstUse() :

   asset.ioSMA = [0.000] * asset.iTabLast

   # -----------------------------------------------------------------------
   asset.SMA = [[ 0.000 for x in range( 0,asset.iTabLast ) ] for y in range( 0,asset.SMAdepth + 1 ) ]

   asset.FirstSMA     = 0
   asset.LastSMA      = asset.SMAdepth - 1
   asset.MaxSMA       = asset.SMAdepth + 1
   asset.sys_DivCount = 0
   DatLines           = 0

   # -----------------------------------------------------------------------
   # --- SMA Prepare LogData ...
   if os.path.exists( asset.avrLogDataFile ) :
      asset.DB = io.ioRead( asset.avrLogDataFile ).split('\n')   # Get the DataBase-File

      try:
         # -----------------------------------------------------------------
         # --- Check if number of values is equal to expected ...
         bFloat = '.' in asset.DB[ 0 ].split()[ 0 ]
         if not bFloat or len( asset.DB[ 0 ].split()) < asset.iTabLast :
            UpdateDB( bFloat, asset.iTabLast )

         # -----------------------------------------------------------------
         # --- Init SMA with the last lines in LogData ...
         DatLines = len( asset.DB )
         if DatLines > asset.SMAdepth : DatLines = asset.SMAdepth
         sList = asset.DB[ len(asset.DB) - DatLines: ]

         for i in range( 0, DatLines ) :
            tmp = [ float ( x ) for x in sList[i].split() ]
            for j in asset.BaseListSMA :
               asset.SMA[ i ][ j ] = tmp[ j ]
      except:
         del asset.DB

   # -----------------------------------------------------------------------
   # --- Fill in missing values ...
   if DatLines != asset.SMAdepth :
      Sensors.Get_CPU ()      # --- CPU-Temperature
      # --------------------------------------------------------------------
      Sensors.Get_Load()      # --- CPU-Load
      Sensors.Get_Fan ()      # --- FAN-Values (PWM-Values) in percent (%)
      Sensors.Get_i2c ()      # --- i2c Form and average the average temperature
      Sensors.Get_Disk()      # --- Disk I/O statistics
      Sensors.Get_Eth ()      # --- Network I/O statistics
      Sensors.Get_Oid ()      # --- OID-Data
      Sensors.Get_Nut ()      # --- UPS Status
      Memory.Get_RAM  ()      # --- RAM usage

   for i in range( DatLines, asset.SMAdepth ) :
      for j in asset.BaseListSMA :
         asset.SMA[ i ][ j ] = asset.SMA[ asset.LastSMA ][ j ]

   asset.Summe = [0.000] * asset.iTabLast

   # -----------------------------------------------------------------------
   # --- Calculate sum for later mean value ...
   for j in asset.BaseListSMA :
      for i in range( 0, asset.SMAdepth ) :
         # -----------------------------------------------------------------
         # --- Remember: The SMA is a floating point number !
         asset.Summe[ j ] += asset.SMA[ i ][ j ]

# --------------------------------------------------------------------------
# --- Init DataBase and Timings
# --------------------------------------------------------------------------
def DataBase():

   try:
      if asset.svg_TimeBase :
         rel.relTimeBase = '{0}{1}{2}{3}{4}'.format( asset.sys_Div, asset.sys_Tick, asset.SMAdepth, asset.svg_TimeBase, asset.svg_TimeStep )
   except:
      pass

    # --- Init query Frequency Table-Pos.
   while switch( asset.AvrConf["LogAccuracy"] ):
      if case(   'high' ):
         DataBaseNum = 0
         break
      if case( 'middle' ):
         DataBaseNum = 1
         break
      if case(    'low' ):
         DataBaseNum = 2
         break
      if case(   'yawn' ):
         DataBaseNum = 3
         break
         # --- Default's ----------------------------------------------------
      DataBaseNum = 2
      assset.AvrConf["LogAccuracy"] = "middle"
      break

   while switch( asset.AvrConf["LogBase"] ):
        #              +---- high ---+ +--- middle --+ +---- low ----+ +--- yawn ----+    
      if case(   '1' ):
         DataBase = [  5,  2,  2,  3,   3,  1,  1,  2,  2,  1,  1,  1,  1,  1,  1,  1,  1 ]
         break
      if case(   '2' ):
         DataBase = [  5,  2,  3,  4,   6,  1,  1,  2,  3,  1,  1,  1,  2,  1,  1,  1,  1 ]
         break
      if case(   '3' ):
         DataBase = [ 10,  3,  4,  6,   9,  2,  3,  3,  4,  1,  1,  2,  3,  1,  1,  1,  2 ]
         break
      if case(   '6' ):
         DataBase = [ 20,  6,  8, 12,  18,  3,  4,  6,  9,  2,  2,  3,  5,  1,  2,  2,  3 ]
         break
      if case(  '12' ):
         DataBase = [ 30, 12, 16, 24,  36,  6,  8, 12, 18,  3,  4,  6,  9,  2,  3,  4,  6 ]
         break
      if case(  '24' ):
         DataBase = [ 60, 24, 32, 48,  72, 12, 16, 24, 36,  6,  8, 12, 18,  4,  5,  8, 12 ]
         break
      if case(  '48' ):
         DataBase = [120, 48, 64, 96, 144, 24, 32, 48, 72, 12, 16, 24, 36,  8, 10, 16, 24 ]
         break
      if case( '168' ):
         DataBase = [420, 48, 64, 96, 144, 24, 32, 48, 72, 24, 32, 48, 72, 18, 24, 36, 54 ]
         break
        # --- Default's ------------------------------------------------------------------
      DataBase = [ 60, 24, 32, 48,  72, 12, 16, 24, 36,  6,  8, 12, 18,  4,  5,  8, 12 ]
      asset.AvrConf["LogBase"] = "24"
      break

   # ----------------------------------------------------------
   # --- Init Multiplier for the Prescaler (DataBase Size)
   while switch( asset.AvrConf["LogDataDiv"] ):
      if case( '1200' ):
         asset.sys_MaxLines = 1200
         asset.pre_MaxLines =  300
         DataBaseDiv        =    1
         break
      if case(  '900' ):
         asset.sys_MaxLines =  900
         asset.pre_MaxLines =  300
         DataBaseDiv        =    2
         break
      if case(  '600' ):
         asset.sys_MaxLines =  600
         asset.pre_MaxLines =  300
         DataBaseDiv        =    3
         break
      if case(  '400' ):
         asset.sys_MaxLines =  400
         asset.pre_MaxLines =  200
         DataBaseDiv        =    4
         break
     # --- Default's ----------------------------------------------------
      asset.sys_MaxLines =  600
      asset.pre_MaxLines =  300
      DataBaseDiv        =    3
      asset.AvrConf["LogDataDiv"] = "600"
      break

   DataBasePos = DataBaseNum * 4 + DataBaseDiv

   asset.sys_Div  = DataBase[DataBasePos]
   asset.sys_Tick = int(asset.AvrConf["LogBase"]) * 3600 / asset.sys_MaxLines / asset.sys_Div

   asset.svg_TimeBase = DataBase[0] * 60
   asset.svg_TimeStep = asset.svg_TimeBase  / asset.sys_Div / asset.sys_Tick

    # ----------------------------------------------------------
    # ---  Init simple moving average (SMA) depth ...
   while switch( asset.AvrConf["LogSMA"] ):
      if case(  '3' ):
         asset.SMAdepth =  3
         break
      if case(  '5' ):
         asset.SMAdepth =  5
         break
      if case( '10' ):
         asset.SMAdepth = 10
         break
      if case( '20' ):
         asset.SMAdepth = 20
         break
      if case( '35' ):
         asset.SMAdepth = 35
         break
      if case( '50' ):
         asset.SMAdepth = 50
         break
      if case( '75' ):
         asset.SMAdepth = 75
         break
      # --- Default's ----------------------------------------------------
      asset.SMAdepth = 10
      asset.AvrConf["LogSMA"]   = 10
      break

   if rel.relTimeBase != '{0}{1}{2}{3}{4}'.format( asset.sys_Div, asset.sys_Tick, asset.SMAdepth, asset.svg_TimeBase, asset.svg_TimeStep ) :
      rel.relBaseList = None
      rel.relTimeBase = None

# --------------------------------------------------------------------------
# --- Check Axis Scaling-Values for validity and initialize scaling parameters 
# --------------------------------------------------------------------------
def MinMax():

   asset.ScalingAxis_Names = [None] * 6
   asset.ScalingAxis_Min   = [None] * 6
   asset.ScalingAxis_Max   = [None] * 6
   asset.ScalingAxis_Def   = [None] * 6
   asset.ScalingAuto_Max   = [None] * 6
   asset.ScalingAuto_Min   = [None] * 6

   asset.ScalingMinMax     = None

   # -------------------------------------------------------------------------------
   # --- Go through all axes and check / initialize values ...
   # -------------------------------------------------------------------------------
   for Tab, Key in enumerate( [ 'Pri1', 'Sec1', 'Pri2', 'Sec2', 'Pri3', 'Sec3' ] ) :

      asset.ScalingAxis_Min[Tab] = int( asset.AvrConf[ Key + "Min"      ] )   # Get Axis Scale-Min
      asset.ScalingAxis_Max[Tab] = int( asset.AvrConf[ Key + "ScaleMax" ] )   # Get Axis Autoscale-Max

      asset.ScalingAuto_Min[Tab] = int( asset.AvrConf[ Key + "ScaleMin" ] )   # Get Axis Autoscale-Min
      asset.ScalingAuto_Max[Tab] = int( asset.AvrConf[ Key + "ScaleMax" ] )   # Get Axis Autoscale-Max

      # -------------------------------------------------------------------------------
      # --- Make sure that the difference between "Axis Scale-Min" and "Axis Scale-Max" is always at least 1 ...
      if asset.ScalingAxis_Max[Tab] <= asset.ScalingAxis_Min[Tab] :
         asset.ScalingAxis_Max[Tab] =  asset.ScalingAxis_Min[Tab] + 1
         asset.AvrConf[Key + "ScaleMax"] = str( asset.ScalingAxis_Max[Tab] )

      # -------------------------------------------------------------------------------
      # --- Make sure that "Autoscale-Min" is always greater / equal then "Axis Scale-Min" ...
      if asset.ScalingAuto_Min[Tab] < asset.ScalingAxis_Min[Tab] :
         asset.ScalingAuto_Min[Tab] = asset.ScalingAxis_Min[Tab]

      # -------------------------------------------------------------------------------
      # --- Make sure that "Autoscale-Max" is always greater / equal then "Autoscale-Min" ...
      if asset.ScalingAuto_Max[Tab] < asset.ScalingAuto_Min[Tab] :
         asset.ScalingAuto_Max[Tab] = asset.ScalingAuto_Min[Tab]
         asset.AvrConf[Key + "ScaleMax"] = str( asset.ScalingAuto_Min[Tab] )

      # -------------------------------------------------------------------------------
      for j in asset.BaseList :
         if asset.BaseNum[j] == Tab :
            try   : asset.ScalingAxis_Names[Tab].append( j )
            except: asset.ScalingAxis_Names[Tab]     = [ j ]

      asset.ScalingAxis_Min[Tab] = int( asset.AvrConf[Key + "Min"] )        # Get Axis Scale-Min
      asset.ScalingAxis_Max[Tab] = int( asset.AvrConf[Key + "Min"] ) + 1    # Get Axis Scale-Max

      # -------------------------------------------------------------------------------
      asset.ScalingAxis_Min[Tab] = asset.ScalingAxis_Min[Tab]
      asset.ScalingAxis_Max[Tab] = asset.ScalingAxis_Max[Tab]
      asset.ScalingAxis_Def[Tab] = asset.ScalingAxis_Max[Tab]

      asset.ScalingAuto_Min[Tab] = asset.ScalingAuto_Min[Tab]
      asset.ScalingAuto_Max[Tab] = asset.ScalingAuto_Max[Tab]

      # -------------------------------------------------------------------------------
      # --- Remember: All Values are integer !      
      Flow.RoundUp( Tab, asset.ScalingAxis_Max[ Tab ] )

      try:    asset.ScalingMinMax += f' {asset.ScalingAxis_Min[Tab]} {asset.ScalingAxis_Max[Tab]}'
      except: asset.ScalingMinMax  =  f'{asset.ScalingAxis_Min[Tab]} {asset.ScalingAxis_Max[Tab]}'

   # ----------------------------------------------------------------------------------
   if not os.path.exists( asset.avrScalingFile ) :
      io.ioWrite( asset.ScalingMinMax + " :initMinMax\n", asset.avrScalingFile )

# ----------------------------------------------------------
# --- Initialize basic values ...
# ----------------------------------------------------------
def BaseList():
   Avr  = asset.AvrConf
   smtp = Status

   try :
      if asset.BaseList :
         rel.relBaseList = ''.join(( ''.joi(( [ str( x ) for x in asset.BaseList ] )), asset.BaseEthList, asset.BaseMdList, Avr["BaseListAlwaysOn"] ))
   except:
      pass

   # ----------------------------------------------------------
#  if smtp.Conf['eventmail1'] != "" or smtp.Conf['eventmail2'] != "" :
   if not True in smtp.eMailMask :
      BaseMail = True
   else :
      BaseMail = False


   # ----------------------------------------------------------
   if asset.AvrConf["StatusPushRegistered"] == "Yes" :
      BasePush = True
   else :
      BasePush = False

   asset.BaseName    = [None] * asset.iTabLast
   asset.BaseNum     = [None] * asset.iTabLast
   asset.BaseDef     = [None] * asset.iTabLast
   asset.BaseSign    = [None] * asset.iTabLast
   asset.cgiBaseName = [None] * asset.iTabLast

   asset.BaseList    = []
   asset.BaseListSMA = []

   stat.AlertName    = [None] * asset.iTabLast
   stat.AlertList    = []


   # ----------------------------------------------------------
   # --- Anordnung der Achsen :
   # --- Primary   = "LINKS" und dann von Innen nach Außen.
   # --- Secondary = "RECHTS"
   # ----------------------------------------------------------
   # --- Primary Axis #1 Temperature °C ...
   # --- Axis Number: 0
   DoBaseList( asset.TabCpu,        "CPU",          0, "Pri1Min", Avr["LegSign"], Avr["PlotCPULabel"],          "CPU °C :",         BaseMail, BasePush )
   DoBaseList( asset.TabStatus,     "Status",       0, "Pri1Min", Avr["LegSign"], Avr["PlotStatusLabel"],       "Thermal Status :", BaseMail, BasePush )
   DoBaseList( asset.TabHmax,       "HddMax",       0, "Pri1Min", Avr["LegSign"], Avr["PlotHddMaxLabel"],       "HDD max. °C :",    BaseMail, BasePush )
   DoBaseList( asset.TabHmin,       "HddMin",       0, "Pri1Min", Avr["LegSign"], Avr["PlotHddMinLabel"],       "HDD min. °C :",    BaseMail, BasePush )
   DoBaseList( asset.TabHavr,       "HddAvr",       0, "Pri1Min", Avr["LegSign"], Avr["PlotHddAvrLabel"],       "HDD avr. °C :",    BaseMail, BasePush )
   DoBaseList( asset.Tabi2c,        "i2c",          0, "Pri1Min", Avr["LegSign"], Avr["Ploti2cLabel"],          "i2c NAS °C :",     BaseMail, BasePush )

   for iID, sID in enumerate( asset.sTabEnv ):
      DoBaseList( asset.iTabEnv[iID], "Ambiente" + sID, 0, "Pri1Min", Avr["LegSign"], Avr["PlotAmbiente%sLabel" % sID], "Ambiente °C %s :" % asset.dTabEnv[ iID ], BaseMail, BasePush )

   # ----------------------------------------------------------
   # --- Secondary Axis #1 Load percent % ...
   # --- Axis Number: 1
   DoBaseList( asset.TabLoad,       "Load",         1, "Sec1Min", Avr["LegSign"], Avr["PlotLoadLabel"],         "System Load :",    BaseMail, BasePush )
   DoBaseList( asset.TabPwm,        "Pwm",          1, "Sec1Min", Avr["LegSign"], Avr["PlotPwmLabel"],          "Fan Speed :",      BaseMail, BasePush )
   DoBaseList( asset.TabUsvLoad,    "UsvLoad",      1, "Sec1Min", Avr["LegSign"], Avr["PlotUsvLoadLabel"],      "USV Load :",       BaseMail, BasePush )
   DoBaseList( asset.TabUsvCharge,  "UsvDischarge", 1, "Sec1Min", Avr["LegSign"], Avr["PlotUsvDischargeLabel"], "USV Discharge :",  BaseMail, BasePush )

   GUI = lib.Split ( asset.msg['SelectRamGUI'] )
   sel = lib.Split ( Avr[ 'RamDeviceListRAW' ] )

   for id in range( 0, len( sel ) ) :
      if sel[id] == Avr["RamSelectDevice"] :
         break

   DoBaseList( asset.TabRAM,    "Memory",   1, "Sec1Min", Avr["LegSign"], Avr["PlotMemoryLabel"],   "RAM: " + GUI[id] + " :",      BaseMail, BasePush )

   # ----------------------------------------------------------
   # --- Primary Axis #2 Network Interface(s) ...
   # --- Axis Number: 2
   asset.sys_EthGetList = dict()
   asset.BaseEthList    = None

   GUI = lib.Split( Avr["EthDeviceListGUI"] )
   sel = lib.Split( Avr["EthDeviceListRAW"] )

   for i in range ( 0, asset.TabEth_Max ) :
      sEth = f'Eth{i}Device'
      if Avr[ sEth ] != "none" :
         for id in range ( 0, len(sel) ) :
            if sel[id] == Avr[ sEth ] :
               break

         tmp = GUI[id].replace( ' ', '' )
         tmp = tmp.replace( '#', '' )
         lr_tmp = ' '.join(( tmp, "receive :"  ))
         lw_tmp = ' '.join(( tmp, "transmit :" ))
      else :
         lr_tmp = "NA"
         lw_tmp = "NA"

      lr = 2 * i + asset.TabEth
      lw = lr + 1

      try:    asset.BaseEthList += " " + Avr[ sEth ]
      except: asset.BaseEthList  =       Avr[ sEth ]

      if Avr[ sEth ] != "none" :
         if Avr[ sEth ] in asset.sys_cmpEthGetList :
            asset.sys_EthGetList[Avr[ sEth ]] = Avr[ sEth ]
         else :
            asset.AvrConf[ sEth ] = "none"

      if Avr[ sEth ] != "none" :
         if Avr["LegIoSignOn"] == "No" :
            DoBaseList( lr, f'Eth{i}r', 2, "Pri2Min", Avr["LegSign"], Avr[ f'PlotEth{i}rLabel' ] + f' (Eth{i}r)', lr_tmp, BaseMail, BasePush )
            DoBaseList( lw, f'Eth{i}w', 2, "Pri2Min", Avr["LegSign"], Avr[ f'PlotEth{i}wLabel' ] + f' (Eth{i}w)', lw_tmp, BaseMail, BasePush )
         else :
            DoBaseList( lr, f'Eth{i}r', 2, "Pri2Min", "DownArrow",    Avr[ f'PlotEth{i}rLabel' ] + f' (Eth{i}r)', lr_tmp, BaseMail, BasePush )
            DoBaseList( lw, f'Eth{i}w', 2, "Pri2Min", "UpArrow",      Avr[ f'PlotEth{i}wLabel' ] + f' (Eth{i}w)', lw_tmp, BaseMail, BasePush )
      else :
         asset.BaseName[lr]    = f'Eth{i}r'
         asset.BaseName[lw]    = f'Eth{i}w'
         asset.cgiBaseName[lr] = f'"{lr_tmp}"'
         asset.cgiBaseName[lw] = f'"{lw_tmp}"'



   # ----------------------------------------------------------
   # --- Secondary Axis #2 Storage Pool device(s) ...
   # --- Axis Number: 3
   asset.TabMdArr        = dict()
   asset.sys_MdSelList_r = dict()
   asset.sys_MdSelList_w = dict()
   asset.sys_MdGetList   = dict()

   asset.BaseMdList = None

   GUI = lib.Split( Avr["MdDeviceListGUI"] )
   sel = lib.Split( Avr["MdDeviceListRAW"] )

   for i in range ( 0, asset.TabMd_Max ) :
      sMd = f'Md{i}Device'
      if Avr[ sMd ] != "none" :
         for id in range ( 0, len(sel) ) :
            if sel[id] == Avr[ sMd ] :
               break

         tmp = GUI[id].replace( ' ', '' )
         tmp = tmp.replace( '#', '' )
         lr_tmp = ' '.join(( tmp, "read :"  ))
         lw_tmp = ' '.join(( tmp, "write :" ))
      else :
         lr_tmp = "NA"
         lw_tmp = "NA"

      lr = 2 * i + asset.TabMD
      lw = lr + 1

      try:    asset.BaseMdList += " " + Avr[ sMd ]
      except: asset.BaseMdList  =       Avr[ sMd ]

      if Avr[ sMd ] != "none" :
         if Avr[ sMd ] in asset.sys_cmpMdGetList :
            asset.sys_MdGetList[Avr[ sMd ]] = Avr[ sMd ]
         else :
            asset.AvrConf[ sMd ] = "none"

      if Avr[ sMd ] != "none" :
         if Avr["LegIoSignOn"] == "No" :
            DoBaseList( lr, f'Md{i}r', 3, "Sec2Min", Avr["LegSign"], Avr[ f'PlotMd{i}rLabel' ] + f' (Md{i}r)', lr_tmp, BaseMail, BasePush )
            DoBaseList( lw, f'Md{i}w', 3, "Sec2Min", Avr["LegSign"], Avr[ f'PlotMd{i}wLabel' ] + f' (Md{i}w)', lw_tmp, BaseMail, BasePush )
         else :
            DoBaseList( lr, f'Md{i}r', 3, "Sec2Min", "DownArrow",    Avr[ f'PlotMd{i}rLabel' ] + f' (Md{i}r)', lr_tmp, BaseMail, BasePush )
            DoBaseList( lw, f'Md{i}w', 3, "Sec2Min", "UpArrow",      Avr[ f'PlotMd{i}wLabel' ] + f' (Md{i}w)', lw_tmp, BaseMail, BasePush )
      else :
         asset.BaseName[lr]    = f'Md{i}r'
         asset.BaseName[lw]    = f'Md{i}w'
         asset.cgiBaseName[lr] = f'"{lr_tmp}"'
         asset.cgiBaseName[lw] = f'"{lw_tmp}"'

   # ----------------------------------------------------------
   # --- Primary Axis #3 Ookla Ping (ms) ...
   # --- Axis Number: 4
   DoBaseList( asset.TabPing,       "Ping",        4, "Pri3Min", Avr["LegSign"], Avr["PlotPingLabel"],      "Ping (ms) :",    BaseMail, BasePush )

   # ----------------------------------------------------------
   # --- Secondary Axis #3 UPS Input Voltage (V) ...
   # --- Axis Number: 5
   DoBaseList( asset.TabUsvInput,   "UsvInput",    5, "Sec3Min", Avr["LegSign"], Avr["PlotUsvInputLabel"],  "USV Volt :",     BaseMail, BasePush )

   # ----------------------------------------------------------
   if rel.relBaseList != ''.join(( ''.join(( [ str( x )for x in asset.BaseList ] )), asset.BaseEthList, asset.BaseMdList, Avr["BaseListAlwaysOn"] )) :
      rel.relBaseList = None

   if Avr["BaseListAlwaysOn"] == "Yes" :
      asset.sys_BaseListAlwaysOn = True
   else :
      asset.sys_BaseListAlwaysOn = False
      asset.BaseListSMA = asset.BaseList

   asset.AvrConf["SvgBaseName"] = ' '.join( asset.BaseName )
   asset.AvrConf["SvgBaseDesc"] = ' '.join( asset.cgiBaseName )

# ----------------------------------------------------------
# --- Sub-Function of "init_BaseList" :
# ----------------------------------------------------------
# --- Build List "AlertList"
# --- Build List "BaseList"
# ----------------------------------------------------------
def DoBaseList( iID, sName, iBase, sBase, sSign, sLabel, sCgi, BaseMail, BasePush ):

   Avr = asset.AvrConf

   if Avr["Plot" + sName + "On"] != "" :
      asset.BaseListSMA += [ iID ]

   asset.BaseName[iID]    = sName
   asset.BaseNum[iID]     = int( iBase )
   asset.BaseDef[iID]     = float( Avr[sBase] )
   asset.cgiBaseName[iID] = ''.join(( '\"', sCgi, '\"' ))

   if Avr["Plot" + sName + "On"] != "Yes" : return

   # ----------------------------------------------------------
   asset.BaseList += [ iID ]

   asset.BaseSign[ iID ]  = sSign
   stat.AlertName[ iID ]  = sLabel

   if Avr["Alert" + sName + "DsmOn"] == "Yes" or ( Avr["Alert" + sName + "MailOn"] == "Yes" and BaseMail ) or ( Avr["Alert" + sName + "PushOn"] == "Yes" and BasePush ) :
      stat.AlertList.append( iID )

      if not stat.AlertTemp[iID] :
         stat.AlertTemp[iID] = int( Avr["Alert" + sName + "Max"] )
      else :
         if stat.AlertTemp[iID] != int( Avr["Alert" + sName + "Max"] ) :
            stat.AlertTemp[iID]  = int( Avr["Alert" + sName + "Max"] )
            stat.AlertWarn[iID]  = None
            stat.AlertTime[iID]  = None
            stat.AlertMax[iID]   = None
   else :
      stat.AlertTemp[iID] = None
      stat.AlertWarn[iID] = None
      stat.AlertTime[iID] = None
      stat.AlertMax[iID]  = None

# --------------------------------------------------------------------------
# --------------------------------------------------------------------------
# sys.exit()
# --------------------------------------------------------------------------
# --------------------------------------------------------------------------
