# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# coding: utf-8                                                                      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#
import os
import sys
import time
from   math     import log10 as log

import asset
import basics   as io
import lib
from   lib      import xrange, switch, case
import Flow     as flow

# -----------------------------------------------------------------------------------
line = sys.modules[ __name__ ]
# -----------------------------------------------------------------------------------
# --- Global options ...
# -----------------------------------------------------------------------------------
iAxis = [      0,      1,      2,     3,       4,     5  ]
sKey  = [ 'Pri1', 'Sec1', 'Pri2', 'Sec2', 'Pri3', 'Sec4' ]

Conf  = dict()
Plot  = dict()

# --------------------------------------------------------------------------
# --- Prepare "Singel Line" SVG-Values
# --------------------------------------------------------------------------
def SVG( iID, iWidth, iHeight, iTop, iLeft, sColor) :

   import Sensors
   import Status
   import Init    as AvrInit

   # --------------------------------------------------------------------------------
   # --- "Always" load the config first ...
   asset.AvrConf = lib.ConfigRead( asset.avrConfigFile )
   # --- Init Language for AvrLogger ...
   asset.msg = lib.InitLanguage( 'admin', asset.avrLangFile )

   # --------------------------------------------------------------------------------
   Sensors.init ()
   Status.init  ()
   AvrInit.DataBase()
   AvrInit.BaseList()
   AvrInit.MinMax  ()

   # --------------------------------------------------------------------------------
   # --- Get the DataBase-File ...
   if os.path.exists( asset.avrLogDataFile ) :
      DB = io.ioRead( asset.avrLogDataFile ).split('\n')

   try:    Lines = len( DB )
   except: Lines = False

   # ----------------------------------------------------------
   # --- Shorten DataBase ...
   if Lines :
      if Lines >= asset.sys_MaxLines :
         DB = DB[ Lines - asset.sys_MaxLines: ]

      iStep = round( asset.sys_MaxLines / asset.pre_MaxLines, 2)
      asset.DB = []

      for a in xrange( iStep-1, len ( DB ), iStep, opt='<=' ) :
         iLine = int( a )
         sDB =  [ '0.000' ] * asset.iTabLast
         sDB[ iID ] = DB[ iLine ].split( ' ' )[ iID ]
         asset.DB.append( ' '.join( sDB ) )

   # ----------------------------------------------------------
   flow.ChangeMin()
   flow.init     ()
   # ----------------------------------------------------------

   # -----------------------------------------------------------------------
   # -----------------------------------------------------------------------
   # --- Get the SVG-Config for Mail-Setup ....
   line.Conf = lib.ConfigRead( asset.MailConfigFile )
   Conf = line.Conf

   # -----------------------------------------------------------------------
   line.SizeWidht       = int( Conf["PageSizeWidht"]  )
   line.SizeHeight      = int( Conf["PageSizeHeight"] )

   line.BoxWidth        = line.SizeWidht
   line.BoxHeight       = line.SizeHeight

   line.BorderEdge      = float( Conf["BorderEdge"] )

   line.BorderX         = line.BorderEdge
   line.BorderY         = line.BorderEdge

   line.BorderWidth     = line.BoxWidth  - ( 2 * line.BorderEdge )
   line.BorderHeight    = line.BoxHeight - ( 2 * line.BorderEdge )

   # -----------------------------------------------------------------------
   # --- Prepare Borders
   # -----------------------------------------------------------------------
   line.PlotEdgeLeft    = float( Conf["PlotEdgeLeft"]   )
   line.PlotEdgeRight   = float( Conf["PlotEdgeRight"]  )
   line.PlotEdgeTop     = float( Conf["PlotEdgeTop"]    )
   line.PlotEdgeBottom  = float( Conf["PlotEdgeBottom"] )

   line.PlotLabelWidth  = float( Conf["PlotLabelWidth"] )
   line.PlotLabelHeight = float( Conf["PlotLabelHeight"])

   line.PlotEdgeLeft    = line.PlotEdgeLeft   + line.PlotLabelWidth
   line.PlotEdgeBottom  = line.PlotEdgeBottom + line.PlotLabelHeight

   line.pw              = line.BoxWidth  - line.PlotEdgeLeft - line.PlotEdgeRight
   line.ph              = line.BoxHeight - line.PlotEdgeTop  - line.PlotEdgeBottom

   # ----------------------------------------------------------
   # --- Prepare SVG-Header
   line.Plot["header"]  = '<div class="svg" style="width: {0}px; height: {1}px; margin-top: {2}px; ;margin-left: {3}px;">\n'.format( iWidth, iHeight, iTop, iLeft )
   line.Plot["header"] += '<svg version="1.2" baseProfile="tiny" id="{0}"'.format( asset.SvgPageID )
   line.Plot["header"] += ' width="100%" height="100%"'
   line.Plot["header"] += ' viewBox="0 0 {0} {1}"'.format( line.BoxWidth, line.BoxHeight )
   line.Plot["header"] += ' overflow="visible"'
   line.Plot["header"] += ' enable-background="new 0 0 {0} {1}"'.format( line.BoxWidth, line.BoxHeight )
   line.Plot["header"] += ' xml:space="preserve">\n'

   # --- Clipping Area ---------------------------------------
   line.Plot["header"] += '\t<defs>\n'
   line.Plot["header"] += '\t\t<clipPath id="PlotArea" clipPathUnits="userSpaceOnUse">\n'
   line.Plot["header"] += '\t\t\t<rect x="-1.0" y="-1.0" width="{0}" height="{1}"/>\n'.format( line.pw + 2.0, line.ph + 2.0 )
   line.Plot["header"] += '\t\t</clipPath>\n'
   line.Plot["header"] += '\t</defs>\n'
    # ---------------------------------------------------------
   line.Plot["header"] += '\t<g id="Page">\n'
   line.Plot["header"] += '\t\t<rect fill="none" stroke="{0}" stroke-width="{1}"'.format( sColor, line.Conf["BorderStroke"] )
   line.Plot["header"] += ' x="{0}" y="{1}" width="{2}" height="{3}"/>\n'.format( line.BorderX, line.BorderY, line.BorderWidth, line.BorderHeight )
   line.Plot["header"] += '\t\t<rect fill="#FFFFFF" stroke="none" stroke-width="1" x="{0}" y="{1}" width="{2}" height="{3}"/>\n'.format( line.PlotEdgeLeft, line.PlotEdgeTop, line.pw, line.ph )

   # ----------------------------------------------------------
   Grid ( 'HorGrid', 'h' )
   Grid ( 'VerGrid', 'v' )
   Label ( iID )

   r = "%.2f" % ( float(line.Conf["PlotLineStroke"]) )
   line.Plot["PlotColor"] = '\t\t\t\t<path id="Mail{0}" vector-effect="non-scaling-stroke" stroke="{1}" stroke-opacity="1.00" stroke-width="{2}"\n'.format( asset.BaseName[iID], line.Conf["PlotLineColor"], r )

   Axis = sKey[ asset.BaseNum[ iID ] ]
   if asset.AvrConf[ Axis + 'ScaleType' ] == 'Lin' : f_Lin = True
   else                                            : f_Lin = False

   if f_Lin : y = line.ph / (      asset.ScalingAxis_Max[ asset.BaseNum[ iID ] ] - asset.ScalingAxis_Min[ asset.BaseNum[ iID] ] )
   else     : y = line.ph / ( log( asset.ScalingAxis_Max[ asset.BaseNum[ iID ] ] - asset.ScalingAxis_Min[ asset.BaseNum[ iID] ] ) )

   x = '%.4f' % ( line.pw / ( asset.pre_MaxLines - 1 ) )
#  y = '%.4f' % ( line.ph / ( asset.ScalingAxis_Max[ asset.BaseNum[ iID ] ] - asset.ScalingAxis_Min[ asset.BaseNum[ iID] ] ) )
   y = '%.4f' % ( y )
   line.Plot['PlotAxis'] = '\t\t\t<g id="Axis-Scale" transform="scale({0},{1})">\n'.format( x, y )

   # ----------------------------------------------------------
   # --- Build "Single Line" SVG ...
   # ----------------------------------------------------------
   # --- Timeline update axis label ...
   TimeLabel( iStep )
   # --- Create the SVG plot ...
   SvgMailBody  = line.Plot["header"]
   # ----------------------------------------
   SvgMailBody += '\t\t<g id="HorizontalAxis">\n'
   SvgMailBody += line.Plot["HorGrid"]
   SvgMailBody += line.Plot["Label"]
   SvgMailBody += '\t\t</g>\n'
   # ----------------------------------------
   SvgMailBody += '\t\t<g id="VerticalAxis">\n'
   SvgMailBody += line.Plot["VerGrid"]
   SvgMailBody += line.Plot["VerLabel"]
   SvgMailBody += '\t\t</g>\n'
   # ----------------------------------------
   x = line.PlotEdgeLeft
   y = ( line.PlotEdgeTop + line.ph ) * -1

   SvgMailBody += '\t\t<g id="DataPlot" style="clip-path: url(#PlotArea);" transform="scale(1,-1) translate({0},{1})" fill="none">\n'.format( x, y )

   # ----------------------------------------
   if flow.minFlag :
      SvgMailBody += line.Plot["PlotAxis"]
      # ----------------------------------------------------------
      SvgMailBody += line.Plot["PlotColor"]
      SvgMailBody += flow.startFlow[iID] + '\n'
      SvgMailBody += flow.plotFlow[iID]
      SvgMailBody += '\t\t\t\t"/>\n'
      # ----------------------------------------------------------
      SvgMailBody += '\t\t\t</g>\n'
   # ----------------------------------------

   SvgMailBody += '\t\t</g>\n'
   # ----------------------------------------
   SvgMailBody += '\t</g>\n'
   SvgMailBody += '</svg>\n'
   SvgMailBody += '</div>\n'

   # --------------------------------------------------------------------------------
   # --- Restore the DataBase-File ...
   if os.path.exists( asset.avrLogDataFile ) :
      asset.DB = io.ioRead( asset.avrLogDataFile ).split('\n')

   flow.init    ()

   # --------------------------------------------------------------------------------
   return SvgMailBody

# ----------------------------------------------------------
# --- Print Time-Axis Labels
# ----------------------------------------------------------
def TimeLabel( iStep ) :

   c = 0
   VerLabelPos = float( line.Conf["VerLabelPos"] )

   VerLine = '\t\t\t<g id="VerLine" stroke-width="0.9" stroke="{0}">\n'.format( line.Conf["LabelColor"] )
   VerText = '\t\t\t<g id="VerText" text-anchor="middle" font-family="\'ArialMT\'" font-size="{0}" fill="{1}">\n'.format( line.Conf["LabelSize"], line.Conf["LabelColor"] )

   Inc = line.pw / ( asset.sys_MaxLines / asset.svg_TimeStep )
   y   = line.PlotEdgeTop + line.ph + VerLabelPos
   y1  = line.PlotEdgeTop + line.ph
   xz  = line.PlotEdgeLeft

   svgTimeline = int( int(time.mktime( time.localtime() )) - int( flow.PlotRow * iStep * asset.sys_Tick * asset.sys_Div ) )

   for i in xrange( 0, asset.sys_MaxLines, asset.svg_TimeStep, '<=' ) :
      t  = time.strftime('%H:%M', time.localtime( svgTimeline + ( c * asset.svg_TimeBase ) ))
      x1 = xz + ( Inc * c )
      x  = x1

      VerLine += '\t\t\t\t<path d="M{0},{1} v{2}"/>\n'.format( '%.2f' % (x1), '%.2f' % (y1), line.Conf["LabelLine"] )
      VerText += '\t\t\t\t<text x="{0}" y="{1}">{2}</text>\n'.format( '%.2f' % (x), '%.2f' % (y), t )

      c += 1

   line.Plot["VerLabel"]  = VerLine
   line.Plot["VerLabel"] += '\t\t\t</g>\n'
   line.Plot["VerLabel"] += VerText
   line.Plot["VerLabel"] += '\t\t\t</g>\n'

# ----------------------------------------------------------
# --- Print Vertical-Axis Label
# ----------------------------------------------------------
def Label( iID ) :

   c           = 0
   LabelMax    = round( asset.ScalingAxis_Max[asset.BaseNum[iID]] , 3 )
   LabelMin    = round( asset.ScalingAxis_Min[asset.BaseNum[iID]] , 3 )
   Range       = LabelMax - LabelMin
   LabelPos    = float( line.Conf["LabelPos"] )
   LabelOffset = float( line.Conf["LabelOffset"] )

   HorLine     = '\t\t\t<g id="HorLine" stroke-width="0.9" stroke="{0}">\n'.format( line.Conf["LabelColor"] )
   HorText     = '\t\t\t<g id="HorText" text-anchor="middle" font-family="\'ArialMT\'" font-size="{0}" fill="{1}">\n'.format( line.Conf["LabelSize"], line.Conf["LabelColor"] )

   # --- Fixed divider of 10
   Step = round( ( LabelMax - LabelMin ) / 10 , 3 )

   x    = line.PlotEdgeLeft - LabelPos
   x1   = line.PlotEdgeLeft - float( line.Conf["LabelLine"] )

   Inc  = line.ph / ( Range / Step )
   yz   = line.PlotEdgeTop + line.ph

   for i in xrange( LabelMin, LabelMax, Step, '<=' ) :
      y1 = yz - c
      y  = y1 + LabelOffset

      HorLine += '\t\t\t\t<path d="M{0},{1} h{2}"/>\n'.format(  '%.2f' % (x1), '%.2f' % (y1), line.Conf["LabelLine"] )

      if i >= 10 : LabelValue = str( int(i) )
      else :       LabelValue = '%.1f' % (i)

      HorText += '\t\t\t\t<text x="{0}" y="{1}">{2}</text>\n'.format( '%.2f' % (x), '%.2f' % (y), LabelValue )
      c = c + Inc

   line.Plot["Label"]  = HorLine
   line.Plot["Label"] += '\t\t\t</g>\n'
   line.Plot["Label"] += HorText
   line.Plot["Label"] += '\t\t\t</g>\n'

# ----------------------------------------------------------
# --- Draw Grid
# ----------------------------------------------------------
def Grid( sName, sAlignment ) :

   c = 0
   a = 0

   while switch( sName ):
      if case( 'HorGrid' ):
         Max  = asset.ScalingAxis_Max[0] / 10
         Min  = asset.ScalingAxis_Min[0] / 10
         # --- Fixed divider of 10
         Step = ( Max - Min ) / 10
         Main = Step
         break
      if case( 'VerGrid' ):
         Max  = asset.sys_MaxLines
         Min  = 0
         Step = asset.svg_TimeStep
         Main = Step
         break
      break

   if sAlignment == "h" :
      Range = Max - Min
      Inc   = line.ph / ( Range / Step )
      h     = line.pw
      x     = line.PlotEdgeLeft
      yz    = line.PlotEdgeTop + line.ph
   else :
      Inc   = line.pw / ( asset.sys_MaxLines / asset.svg_TimeStep )
      h     = line.ph
      y     = line.PlotEdgeTop
      xz    = line.PlotEdgeLeft

   line.Plot[sName] = '\t\t\t<g id="{0}" fill="none" stroke-width="{1}" stroke="{2}">\n'.format( sName, line.Conf["GridStroke"], line.Conf["GridColor"])

   for i in xrange( Min, Max, Step, '<=' ) :
      if sAlignment == "h" :
         y = yz - c
      else :
         x = xz + c

      line.Plot[sName] += '\t\t\t\t<path d="M{0},{1} {2}{3}"/>\n'.format( '%.2f' % (x), '%.2f' % (y), sAlignment, '%.2f' % (h) )
      c = c + Inc

   line.Plot[sName] += '\t\t\t</g>\n'

# --------------------------------------- END ---------------------------------------
# -----------------------------------------------------------------------------------
#  sys.exit()
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
