# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# coding: utf-8                                                                      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#

import os
import time

import asset
import basics   as io

import lib
import Init as rel

# -----------------------------------------------------------------------------------
# --- Global options ...
# -----------------------------------------------------------------------------------
import __main__ as avr

walkError = {
               'Timeout'            : 'Unknown host',                                              # (v2) Timeout: No Response from 192.168.1.48
               'Unknown community'  : 'Unknown community string',                                  # (v2) Error: Unknown community string
               'Unknown host'       : 'Unknown host',                                              # (v2) Error: Unknown host
               'No Such Object'     : 'No Such Object',                                            # (v2) No Such Object available on this agent at this OID
               'No Such Instance'   : 'No Such Instance',                                          # (v2) No Such Instance currently exists at this OID
               'No Response'        : 'No Response from host',
               'Unknown engine'     : 'Unknown engine ID (Permission denied)',
               'Unknown user name'  : 'Unknown user name',                                         # (v3) snmpwalk: Unknown user name
               'Authentication'     : 'Authentication failure. Check password and encryption',     # (v3) snmpwalk: Authentication failure (incorrect password, community or key)
               'snmpwalk: Timeout'  : 'No Response from host',                                     # (v3) snmpwalk: Timeout
            }

# -----------------------------------------------------------------------------------
# --- Query a Environmental sensor ...
# -----------------------------------------------------------------------------------
def Test( Avr ) :

   from Html import Colorize

   # --------------------------------------------------------------------------------
   sID     = asset.sTabEnv[ int( Avr['LogEnvCurrent'] ) ]

   if Avr[ 'LogEnvV3On' + sID ] == 'No' : iV2 = True
   else                                 : iV2 = False

   walk    = 'Start'
   Err     = False
   TimeOut = int( time.time() ) +  5

   # --------------------------------------------------------------------------------
   # --- Wait until Query returnes a valid result or TimeOut ...
   while not walk.isdigit() :
      if int( time.time() ) >= TimeOut :
         # --------------------------------------------------------------------------
         if 'Timeout' in walk :
            # --- Run a ping to test Agend's-IP ...
            if os.system( 'ping -c 1 ' + Avr[ 'LogEnvIp' + sID ] + ' &> /dev/null' ) :
               Err = "Timeout: Unknown host"
            else :
               Err = walk
         else :
            Err = walk
         # --------------------------------------------------------------------------
         break

      # -----------------------------------------------------------------------------
      if walk != 'Start' : time.sleep( 0.333 )

      # -----------------------------------------------------------------------------
      # --- Perform query ...
      if iV2 : walk = lib.WalkV2( Avr[ 'LogEnvCom'  + sID ],                                                             Avr[ 'LogEnvIp' + sID ], Avr[ 'LogEnvOID' + sID ] )
      else   : walk = lib.WalkV3( Avr[ 'LogEnvUser' + sID ], Avr[ 'LogEnvEncryption' + sID ], Avr[ 'LogEnvPass' + sID ], Avr[ 'LogEnvIp' + sID ], Avr[ 'LogEnvOID' + sID ] )

   # --------------------------------------------------------------------------------
   if Err : return '{0} : {1}'.format        ( Colorize( 'Failed  ', 'Gold' ),
                                                 Colorize( Err, 'Red', True )
                                               )
   else   : return '{0} : {1} = {2}°C'.format( Colorize( 'Responce', 'Gold' ),
                                                 Colorize( '{0:>3}'.format( walk ), 'Lime' ),
                                                 Colorize( '{0: >5.1f}'.format( float( walk ) / int( Avr['LogEnvScale' + sID] ) ), 'DodgerBlue', True )
                                               )

# --------------------------------------------------------------------------
# --- Check availability of Environmental sensor
# --------------------------------------------------------------------------
def init( BootInit ):

   Avr = asset.AvrConf

   # --------------------------------------------------------------------------
   """
   if rel.relEnvironmental == ''.join(( Avr["LogEnvOn"],         Avr["LogEnvCom"],  Avr["LogEnvV3On"],    \
                                        Avr["LogEnvEncryption"], Avr["LogEnvUser"], Avr["LogEnvPass"],    \
                                        Avr["LogEnvIp"],         Avr["LogEnvOID"],  Avr["LogEnvScale"] )) :
         return

   rel.relEnvironmental =     ''.join(( Avr["LogEnvOn"],         Avr["LogEnvCom"],  Avr["LogEnvV3On"],    \
                                        Avr["LogEnvEncryption"], Avr["LogEnvUser"], Avr["LogEnvPass"],    \
                                        Avr["LogEnvIp"],         Avr["LogEnvOID"],  Avr["LogEnvScale"] ))
   """

   # --------------------------------------------------------------------------
   for index, sID in enumerate( asset.sTabEnv ):
      if Avr[ 'LogEnvOn' + sID ] != "Yes" :
         asset.AvrConf[ 'PlotAmbiente' + sID + 'On' ] = "No"
         continue

      # -----------------------------------------------------------------------
      if avr.AvrDebug : TimeOut = int( time.time() ) +  5
      else            : TimeOut = int( time.time() ) + 60
      if Avr[ 'LogEnvV3On' + sID ] == 'No' :
         walk = lib.WalkV2( Avr[ 'LogEnvCom' + sID ], Avr[ 'LogEnvIp' + sID ], Avr[ 'LogEnvOID' + sID ] )
      else :
         walk = lib.WalkV3( Avr[ 'LogEnvUser' + sID ], Avr[ 'LogEnvEncryption' + sID ], Avr[ 'LogEnvPass' + sID ], Avr[ 'LogEnvIp' + sID ], Avr[ 'LogEnvOID' + sID ] )

      # -----------------------------------------------------------------------
      # --- Wait until Query returnes a valid result or TimeOut ...
      while not walk.isdigit() :
         io.touch( asset.f_Heartbeat )
         if int( time.time() ) >= TimeOut :
            asset.AvrConf[ 'LogEnvOn' + sID ]            = "No"
            asset.AvrConf[ 'PlotAmbiente' + sID + 'On' ] = "No"
            # -----------------------------------------------------------------
            if 'Timeout' in walk :
               # --- Run a ping to test Agend's-IP ...
               if os.system( 'ping -c 1 ' + Avr[ 'LogEnvIp' + sID ] + ' &> /dev/null' ) :
                  walk = "Error: Unknown host"               # --- "ERROR: Unknown host"
               else :
                  walk = "Error: Unknown community string"   # --- "ERROR: Unknown community string"

            # -----------------------------------------------------------------
            Err = ''.join(( 'ENV: \"', Avr[ 'LogEnvName' + sID ], '\"' ))

            if 'Unknown community' in walk : Err += ''.join(( asset.msg['errEnvString'],     '\"', Avr[ 'LogEnvCom'  + sID ], '\"' ))
            if 'Unknown user name' in walk : Err += ''.join(( asset.msg['errEnvUser'],       '\"', Avr[ 'LogEnvUser' + sID ], '\"' ))
            if 'Unknown host'      in walk : Err += ''.join(( asset.msg['errEnvHost'],       '\"', Avr[ 'LogEnvIp'   + sID ], '\"' ))
            if 'No Such Object'    in walk : Err += ''.join(( asset.msg['errEnvObject'],     '\"', Avr[ 'LogEnvOID'  + sID ], '\"' ))
            if 'No Such Instance'  in walk : Err += ''.join(( asset.msg['errEnvInstance'],   '\"', Avr[ 'LogEnvOID'  + sID ], '\"' ))
            if 'Authentication'    in walk : Err += ''.join(( asset.msg['errEnvPassword']                                          ))
            if 'No Response'       in walk : Err += ''.join(( asset.msg['errEnvNoResponse'], '\"', Avr[ 'LogEnvIp'   + sID ], '\"' ))

            lib.Error( Err )
            # -----------------------------------------------------------------
            break

         # --------------------------------------------------------------------
         if BootInit and not avr.AvrDebug : time.sleep( 1.000 )
         else :                             time.sleep( 0.333 )

         if Avr[ 'LogEnvV3On' + sID ] == 'No' :
            walk = lib.WalkV2( Avr[ 'LogEnvCom' + sID ], Avr[ 'LogEnvIp' + sID ], Avr[ 'LogEnvOID' + sID ] )
         else :
            walk = lib.WalkV3( Avr[ 'LogEnvUser' + sID ], Avr[ 'LogEnvEncryption' + sID ], Avr[ 'LogEnvPass' + sID ], Avr[ 'LogEnvIp' + sID ], Avr[ 'LogEnvOID' + sID ] )

# --------------------------------------------------------------------------
# --------------------------------------------------------------------------
# sys.exit()
# --------------------------------------------------------------------------
# --------------------------------------------------------------------------
