# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# coding: utf-8                                                                      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#
import os

# -----------------------------------------------------------------------------------
# --- Set of "Global" definitions like Pathes, Files, Flags and other stuff ...
# -----------------------------------------------------------------------------------
SpkHome              = os.getcwd()
SpkName              = "AvrLogger"
SpkDaemon            = "AvrLogger"
SpkVersion           = "4.2.0-0404"
SpkBuild             = "2022.04.04 18:51:57"

if SpkHome == '/volume1/homes/admin/AvrLoggerSPK' :
   DsmVersion        = '6'
else :
   DsmVersion        = "7"

cssVersion           = "20220404185157"
SmartVersion         = "200703"

cgiNameConfig        = "AvrLogger"
cgiNameChart         = "AvrChart"

if DsmVersion == "6" : f_Dsm7      = False
else                 : f_Dsm7      = True

# -----------------------------------------------------------------------------------
ETC                  = f'/var/packages/{SpkName}/etc'
PKG                  = f'/var/packages/{SpkName}/target/ui'
ERROR                = f'{ETC}/log/error'

# -----------------------------------------------------------------------------------
# --- Globale (CGI & Deamon ) - definitions :
AvrConf              = dict()    # Global Config
AvrInfo              = dict()
SynoInfo             = dict()    # Content of /etc/synoinfo.conf
msg                  = dict()    # Global Language

# -----------------------------------------------------------------------------------
i18nLangFile         = f'{PKG}/texts/'

sysSynoConfig        = '/etc/synoinfo.conf'
sysSynoGroups        = '/etc/group'
sysUpsConfig         = '/usr/syno/etc/ups/synoups.conf'
sysVpnClient         = '/usr/syno/etc/synovpnclient'
sysUsbMap            = '/usr/syno/etc/usbno_guid.map'
sysVpnServer         = "/usr/syno/etc/packages/VPNCenter"
sysAdminNotification = "/usr/syno/synoman/webman/modules/DisableAdminNotification/config"


avrInfoFile          = f'/var/packages/{SpkName}/INFO'
avrRootText          = f'{PKG}/templates/root.txt'
avrPidFile           = f'{PKG}/{SpkName}.pid'
avrLangFile          = f'{PKG}/lang/lang_'
avrTemplateFiles     = f'{PKG}/templates'
avrPresetFiles       = f'{PKG}/templates/Layout'
avrUsbIdsFiles       = f'{PKG}/templates/usb.ids'

avrEnumScan          = ETC + "/log/enumScan"    # Current Device-List & TimesStamps
avrEnumSmart         = ETC + "/log/enumSmart"   # Current Device-Infos
avrEnumDrives        = ETC + "/log/enumDrives"  # Device Database : Serials; Timestamp; History
avrEnumPool          = ETC + "/log/enumPool"
avrEnumStorage       = ETC + "/log/enumStorage"
avrCpuScore          = ETC + "/log/cpuScore"
LogSearchFile        = ETC + "/log/LogSearch"
LogSearchMeta        = ETC + "/log/LogSearch.json"
avrImagesFiles       = PKG + "/images"

avrConfigFile        = ETC + "/config/log.conf"
avrConfigJson        = ETC + "/config/log.json"
avrLogDataFile       = ETC + "/tmp/" + SpkDaemon + ".dat"

avrUpTimeFile        = ETC + "/log/UpTime"
avrScalingFile       = ETC + "/log/AxisScaling"

f_VirtualDSM         = ETC + "/log/VirtualDSM"
f_ExcludeSCEMD       = ETC + "/log/ExcludeSCEMD"
f_NewsAvrPKG         = ETC + "/log/NewsAvrPKG"

f_AvrDisabled        = ETC + "/log/disabled"
f_SpkEnabled         = "/var/packages/" + SpkName + "/enabled"

f_RestoreConf        = ETC + "/log/RestoreConf"
f_UpgradePKG         = ETC + "/log/UpgradePKG"
f_Heartbeat          = ETC + "/tmp/Heartbeat"
f_AvrLock            = ETC + "/tmp/AvrLock"
f_SenLock            = ETC + "/tmp/SenLock"
f_SvgLock            = ETC + "/tmp/SvgLock"
f_CrnFlag            = ETC + "/tmp/CrnFlag"
f_ExcludeSHM         = ETC + "/log/ExcludeSHM"
f_ShmLock            = ETC + "/log/ShmLock"
f_CgiUpdate          = ETC + "/tmp/CgiUpdate"
f_avrStatus          = ETC + "/tmp/avrStatus"

cgiNavIcon           = [ 'Status_Play_Stop.gif', 'Status_On.gif', 'Status_Off.gif' ]
SynoDiskEnum         = { 'internal' : 0, 'ebox'     : 1, 'cache'    : 3, 'usb'      : 4 }
# -----------------------------------------------------------------------------------
# --- Modul "Sensors" - NUT (UPS) - definitions :
# -----------------------------------------------------------------------------------

nutDefaultHostIP     = '127.0.0.1'
nutDefaultHostPort   = 3493
nutSockTimeOut       = 5.0

# -----------------------------------------------------------------------------------
# --- CGI - definitions :
# -----------------------------------------------------------------------------------
cgiInput             = dict()
Out                  = dict()
avrWhatsNewFile      = PKG + "/templates/WhatsNew_"
avrRootInfo          = PKG + "/templates/Root_"
callAvrReport        = PKG + '/AvrShedule.py mail TestMail'

cgiSynoCheck         = ETC + "/tmp/synoCheck.json"
cgiModulPath         = f'{PKG}/gui'

# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
# --- Modul "Push" - definitions :
# -----------------------------------------------------------------------------------
u_PushBullet_Dev     = 'https://api.pushbullet.com/v2/devices'    # Request all Devices
u_Pushbullet_User    = 'https://api.pushbullet.com/v2/users/me'   # Request current User
u_PushBullet_Note    = 'https://api.pushbullet.com/v2/pushes'     # Performe a Push

# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
# --- Modul "Scemd" - definitions :
# -----------------------------------------------------------------------------------
ScemdFile            = "/usr/syno/etc.defaults/scemd.xml"
ScemdVrmFile         = PKG + "/templates/scemd.xml"

ScemdPatFile         = ETC + "/log/scemd.xml"
ScemdMd5File         = ETC + "/log/scemd.md5"

ScemdBaseURL         = 'https://global.download.synology.com/download/DSM/release'

ScemdModeList        = [ 'HIGH', 'LOW', 'LOW_STOP' ]
ScemdDevList         = [ 'disk', 'm2', 'cpu' ]
ScemdDevNice         = { 'disk' :' Disk', 'm2' : 'NVMe', 'cpu' : 'CPU' }

ScemdPwmList         = [ 'full_speed', 'alert_temp', 'turbo_temp', 'underclock_temp', 'shutdown_temp' ]
ScemdPwmShort        = { 'full_speed' : 'full', 'alert_temp' : 'alert', 'turbo_temp' : 'turbo', 'underclock_temp' : 'underclock', 'shutdown_temp' : 'shutdown' }
ScemdPwmNice         = { 'full' : 'Full-speed' , 'alert' : 'Alarm' , 'turbo' : 'Turbo' , 'underclock' : 'Throttle' , 'shutdown' : 'Shutdown' }

ScemdSynoFanSpeed    = { 'fullfan' : 'FULL', 'coolfan' : 'HIGH', 'quietfan' : 'LOW', 'quietstopfan' : 'LOW_STOP' }
# -----------------------------------------------------------------------------------
# --- SYNO.Core.Hardware.FanSpeed :
# --- In einer virtuellen Umgebung wird kein Wert zurückgegeben
# -----------------------------------------------------------------------------------
# --- 'fullfan'      =  "FULL"     - "Modus volle Gewschwindigkeit" <<< Diesen Modus / dieses Profil gibt es in der scemd.xml nicht !!!
# --- 'coolfan'      =  "HIGH"     - "Kühlmodus"
# --- 'quietfan'     =  "LOW"      - "Stiller Modus"
# --- 'quietstopfan' =  "LOW_STOP" - "Niedriger Energiestatus"

# -----------------------------------------------------------------------------------
AdtSpeedList         = "00 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 75 80 85 90 95 99"
AdtTempList          = "0 10 20 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 95 100 105 110"
PwmTempList          = "45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100"
f_ScemdError         = ETC + "/tmp/ScemdError"
f_ScemdChanged       = ETC + "/tmp/ScemdChanged"
f_ScemdInit          = ETC + "/tmp/ScemdInit"
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
# --- Modul "Speed" - definitions :
# -----------------------------------------------------------------------------------
SpeedLogFile         = PKG + "/templates/speed.log"       # --- Plain-Text Template

SpeedServerXML       = ETC + "/log/speed.xml"
SpeedJsonFile        = ETC + "/log/speed.json"
SpeedResultFile      = ETC + "/log/speed.dat"
SpeedOIDFile         = ETC + "/tmp/speed.oid"

f_SpeedBusy          = ETC + "/tmp/SpeedBusy"
f_SpeedFlag          = ETC + "/tmp/SpeedFlag"
f_FlowFlag           = ETC + "/tmp/FlowFlag"

# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
# --- Modul "Status" & "Line" (Alert / eMail) definitions ...
# -----------------------------------------------------------------------------------
SynoSmtpFile         = "/usr/syno/etc/synosmtp.conf"
MailConfigFile       = ETC + "/config/mail.conf"

# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
# --- Modul "Memory" - definitions :
# -----------------------------------------------------------------------------------
sysMemorySizeFile    = ETC + "/log/MemorySize"

# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
# --- Modul "Cron" - definitions :
# -----------------------------------------------------------------------------------
CronTabFile          = "/etc/cron.d/crontab"
CronConfFile         = ETC + "/log/schedule.conf"

if f_Dsm7 : CronBase = "root\t/usr/bin/python3 "         + PKG
else      : CronBase = "root\t/usr/local/bin/python3 "   + PKG

CronJob              = CronBase + "/AvrShedule.py"

CronScript           = PKG + "/AvrShedule.py"
OoklaScript          = PKG + "/lib/Ookla.py"

# -----------------------------------------------------------------------------------
script               = "/var/packages/" + SpkDaemon + "/scripts/start-stop-status"    # start-stop-status Script

# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
# --- Modul "Oid" definitions ...
# -----------------------------------------------------------------------------------
avrOID_DataFile      = ETC + "/tmp/Oid.dat"
f_OidLock            = ETC + "/tmp/OidLock"
f_OidFlag            = ETC + "/tmp/OidFlag"

# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
# --- Modul "lib" - definitions :
# ----------------------------------------------------------
# timerLap       = None
# timerCount     = None
timerStart     = []
# ----------------------------------------------------------

rnd_SMA        = 6
rnd_ioSMA      = 6
rnd_Summe      = 6
rnd_DB         = 3
rnd_SVG        = 3

# -----------------------------------------------------------------------------------
# ----------------------------------------------------------
# --- Init / build list of used Sesors & ReScale Axis
# ----------------------------------------------------------
TabCnt         =  0
TabMd_Max      =  5   # max. 5x storage devices (HDD or Storage-Pool)
TabEth_Max     =  5   # max. 5x network interfaces (eth or ovs_bond)

TabLoad        = TabCnt; TabCnt += 1 #  0   # AvrLoad              -> Pri #1 - %
TabCpu         = TabCnt; TabCnt += 1 #  1   # AvrCpu               -> Sec #1 -°C
TabStatus      = TabCnt; TabCnt += 1 #  2   # AvrStatus            -> Sec #1 -°C
TabEnv         = TabCnt; TabCnt += 1 #  3   # AvrEnvirenmentSensor -> Sec #1 -°C
TabHmax        = TabCnt; TabCnt += 1 #  4   # AvrHddMax            -> Sec #1 -°C
TabHmin        = TabCnt; TabCnt += 1 #  5   # AvrHddMin            -> Sec #1 -°C
TabHavr        = TabCnt; TabCnt += 1 #  6   # AvrHddAvr  : Average -> Sec #1 -°C
TabPwm         = TabCnt; TabCnt += 1 #  7   # AvrPwm               -> Pri #1 - %
Tabi2c         = TabCnt; TabCnt += 1 #  8   # Avri2c               -> Sec #1 -°C
TabRAM         = TabCnt; TabCnt += 1 #  9   # AvrRAM usage         -> Pri #1 - %
TabPing        = TabCnt; TabCnt += 1 # 10   # Speedtest Ping       -> Pri #2 - eht I/O MiB/Sec
TabMD          = TabCnt; TabCnt += ( TabMd_Max  * 2 ) # 11 # >> 5 * Couple of 'TabMDr / TabMDw'
TabEth         = TabCnt; TabCnt += ( TabEth_Max * 2 ) # 21 # >> 5 * Couple of 'TabEthr / TabEthw'
TabUsvLoad     = TabCnt; TabCnt += 1 # 31
TabUsvCharge   = TabCnt; TabCnt += 1 # 32
TabEnv1        = TabCnt; TabCnt += 1 # 33
TabEnv2        = TabCnt; TabCnt += 1 # 34
TabEnv3        = TabCnt; TabCnt += 1 # 35
TabEnv4        = TabCnt; TabCnt += 1 # 36
TabUsvInput    = TabCnt              # 37

TabLast        = TabCnt; TabCnt += 1
iTabLast       = TabCnt              # 38 <<<<
# ----------------------------------------------------------
iTabEnv        = [ TabEnv, TabEnv1, TabEnv2, TabEnv3, TabEnv4 ]
sTabEnv        = [     '',     '1',     '2',     '3',     '4' ]
dTabEnv        = [  '(1)',   '(2)',   '(3)',   '(4)',   '(5)' ]

# ----------------------------------------------------------

# -----------------------------------------------------------------------------------
# --- "BaseList" definitions ...
AlertTemp      = [None] * iTabLast
AlertWarn      = [None] * iTabLast
AlertTime      = [None] * iTabLast
AlertMax       = [None] * iTabLast

Axis           = [ 0, 1, 2, 3, 4, 5]
# -----------------------------------------------------------------------------------

# -----------------------------------------------------------------------------------
# --- Set of "Global" definitions for Module "Svg" ...
# -----------------------------------------------------------------------------------
SvgVersion           = "22.02.12"
SvgEngine            = f'AvrLogger v{SpkVersion} SVG-Engine v{SvgVersion} : https://www.luenepiet.de/my_avrlogger/main'
SvgPageID            = "AVRlog"

LineChart            = f'{PKG}/images'

SvgPreMax            = 200
SvgLineMax           = 300

svgLineChart         = f'{ETC}/tmp/tmp.svg'
SvgTmp               = f'{ETC}/tmp/svg.tmp'           # SVG-Grafic File
SvgPlot              = f'{ETC}/tmp/tmp.svg'           # SVG-Grafic File
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------

def gui() :

   global objModule
   global desModule

   global PageName
   global FuncSubList
   global Func


   global GuiMail
   global eMailMask
   global eMailList

